@@ -9,10 +9,13 @@
 import os
 import sys
 
-def main(argv):
+def main():
     import quodlibet
     from quodlibet import util
     from quodlibet import const
+    import gobject
+
+    quodlibet._init_signal()
 
     opts = util.OptionParser(
         "Ex Falso", const.VERSION,
@@ -24,14 +27,30 @@ def main(argv):
 
     from quodlibet import config
     config.init(const.CONFIG)
-    backend, library, player = quodlibet.init(icon="exfalso", backend="nullbe")
+
+    library = quodlibet.init(icon="exfalso",
+                             name="Ex Falso",
+                             title=const.PROCESS_TITLE_EF)
+
+    player = quodlibet.init_backend("nullbe", library.librarian)
 
     from quodlibet.qltk.exfalsowindow import ExFalsoWindow
+    window = ExFalsoWindow(library, args[0])
+
+    quodlibet.enable_periodic_save(save_library=False)
+
+    from quodlibet.qltk import session
+    session.init("exfalso", window)
+
     from quodlibet import widgets
-    widgets.main = ExFalsoWindow(library, args[0])
-    quodlibet.main(widgets.main)
-    quodlibet.quit((backend, library, player))
-    config.write(const.CONFIG)
+    widgets.main = window
+    widgets.watcher = library.librarian
+
+    quodlibet.main(window)
+
+    config.save(const.CONFIG)
+
+    print_d("Finished shutdown.")
 
 if __name__ == "__main__":
-    main(sys.argv)
+    main()
