@@ -20,6 +20,7 @@ except ImportError:
 from distutils.command.build import build as distutils_build
 from distutils.command.install import install as distutils_install
 
+from gdist.core import GCommand
 from gdist.gobject import build_gobject_ext, GObjectExtension
 from gdist.shortcuts import build_shortcuts, install_shortcuts
 from gdist.man import install_man
@@ -27,6 +28,11 @@ from gdist.po import build_mo, install_mo
 
 import os
 
+class build_icon_cache(GCommand):
+    """Update the icon theme cache"""
+    def run(self):
+        self.spawn(['gtk-update-icon-cache', '-f', 'quodlibet/images/hicolor'])
+
 class build(distutils_build):
     """Override the default build with new subcommands."""
     sub_commands = distutils_build.sub_commands + [
@@ -36,13 +42,15 @@ class build(distutils_build):
          lambda self: self.distribution.has_shortcuts()),
         ("build_gobject_ext",
          lambda self: self.distribution.has_gobject_ext()),
+        ("build_icon_cache",
+         lambda self: self.distribution.need_icon_cache()),
         ]
 
 class install(distutils_install):
     """Override the default install with new subcommands."""
 
     sub_commands = distutils_install.sub_commands + [
-        ("install_shortcuts", lambda self: self.distribution.has_shortcuts()), 
+        ("install_shortcuts", lambda self: self.distribution.has_shortcuts()),
         ("install_man", lambda self: self.distribution.has_man_pages()),
         ("install_mo", lambda self: self.distribution.has_po()),
        ]
@@ -66,7 +74,7 @@ class GDistribution(Distribution):
     Example:
       from distutils.core import setup
       from gdist import GDistribution
-      
+
       setup(distclass=GDistribution, ...)
       """
 
@@ -81,6 +89,7 @@ class GDistribution(Distribution):
         self.cmdclass.setdefault("build_gobject_ext", build_gobject_ext)
         self.cmdclass.setdefault("build_mo", build_mo)
         self.cmdclass.setdefault("build_shortcuts", build_shortcuts)
+        self.cmdclass.setdefault("build_icon_cache", build_icon_cache)
         self.cmdclass.setdefault("install_shortcuts", install_shortcuts)
         self.cmdclass.setdefault("install_man", install_man)
         self.cmdclass.setdefault("install_mo", install_mo)
@@ -99,4 +108,7 @@ class GDistribution(Distribution):
     def has_man_pages(self):
         return os.name != 'nt' and bool(self.man_pages)
 
+    def need_icon_cache(self):
+        return os.name != 'nt'
+
 __all__ = ["GDistribution", "GObjectExtension"]
