@@ -12,39 +12,39 @@ import os
 import signal
 import sys
 import tempfile
-import time
 
 import quodlibet
-import quodlibet.player
 
 from quodlibet import config
 from quodlibet import const
 from quodlibet import util
 from quodlibet.util.uri import URI
 
-from threading import Thread
-
-global play
 play = False
 
 def main():
+    import quodlibet.player
+
     config.init(const.CONFIG)
 
-    try:
-        backend, library, player = quodlibet.init(
-            backend=config.get("player", "backend"),
-            library=const.LIBRARY,
-            )
-    except quodlibet.player.error, error:
-        import gobject
-        import gtk
-        gobject.idle_add(quodlibet.error_and_quit, error)
-        gobject.idle_add(gtk.main_quit)
-        gtk.main()
-        config.write(const.CONFIG)
-        raise SystemExit(True)
+    library = quodlibet.init(library=const.LIBRARY,
+                             icon="quodlibet",
+                             name="Quod Libet",
+                             title=const.PROCESS_TITLE_QL)
 
-    from quodlibet import widgets
+    os.environ["PULSE_PROP_media.role"] = "music"
+    os.environ["PULSE_PROP_application.icon_name"] = "quodlibet"
+
+    for backend in [config.get("player", "backend"), "nullbe"]:
+        try:
+            player = quodlibet.init_backend(backend, library.librarian)
+        except quodlibet.player.error, error:
+            print_e("%s. %s" % (error.short_desc, error.long_desc))
+        else:
+            break
+
+    from quodlibet import browsers
+    from quodlibet.qltk.songlist import SongList
 
     try: ratings = config.getint("settings", "ratings")
     except (ValueError, TypeError): pass
@@ -54,15 +54,93 @@ def main():
     except (ValueError, TypeError): pass
     else: const.DEFAULT_RATING = default_rating
 
-    window = widgets.init(player, library)
-    if "--debug" not in sys.argv:
-        enable_periodic_save(library)
+    if config.get("settings", "headers").split() == []:
+       config.set("settings", "headers", "title")
+    headers = config.get("settings", "headers").split()
+    SongList.set_all_column_headers(headers)
+
+    for opt in config.options("header_maps"):
+        val = config.get("header_maps", opt)
+        util.tags.add(opt, val)
+
+    in_all =("~filename ~uri ~#lastplayed ~#rating ~#playcount ~#skipcount "
+             "~#added ~#bitrate ~current ~#laststarted ~basename "
+             "~dirname").split()
+    for Kind in browsers.browsers:
+        if Kind.headers is not None: Kind.headers.extend(in_all)
+        Kind.init(library)
+
+    window = init_plugins(player, library)
+
+    from quodlibet.qltk.remote import FSInterface, FIFOControl
+    from quodlibet.qltk.tracker import SongTracker
+    try:
+        from quodlibet.qltk.dbus_ import DBusHandler
+    except ImportError:
+        DBusHandler = lambda player, library: None
+
+    FSInterface(player)
+    FIFOControl(library, window, player)
+    DBusHandler(player, library)
+    SongTracker(library.librarian, player, window.playlist)
+
     if play:
         player.paused = False
+
+    quodlibet.enable_periodic_save(save_library=True)
+
+    from quodlibet.qltk import session
+    session.init("quodlibet", window)
+
     quodlibet.main(window)
-    quodlibet.quit((backend, library, player), save=True)
-    try: config.write(const.CONFIG)
-    except EnvironmentError, err: pass
+
+    print_d("Shutting down player device %r." % player.version_info)
+    quodlibet.player.quit(player)
+
+    quodlibet.library.save(force=True)
+    config.save(const.CONFIG)
+
+    print_d("Finished shutdown.")
+
+def init_plugins(player, library):
+    from quodlibet.plugins.editing import EditingPlugins
+    from quodlibet.plugins.songsmenu import SongsMenuPlugins
+    from quodlibet.plugins.events import EventPlugins
+    from quodlibet.plugins.playorder import PlayOrderPlugins
+    from quodlibet.qltk.songsmenu import SongsMenu
+    from quodlibet.qltk.properties import SongProperties
+    from quodlibet import widgets
+
+    widgets.watcher = library.librarian
+
+    SongsMenu.plugins = SongsMenuPlugins(
+        [os.path.join(const.BASEDIR, "plugins", "songsmenu"),
+         os.path.join(const.USERDIR, "plugins", "songsmenu")], "songsmenu")
+    SongsMenu.plugins.rescan()
+
+    SongProperties.plugins = EditingPlugins(
+        [os.path.join(const.BASEDIR, "plugins", "editing"),
+         os.path.join(const.USERDIR, "plugins", "editing")], "editing")
+
+    playorder = PlayOrderPlugins(
+        [os.path.join(const.BASEDIR, "plugins", "playorder"),
+         os.path.join(const.USERDIR, "plugins", "playorder")], "playorder")
+    playorder.rescan()
+
+    # main window
+    from quodlibet.qltk.quodlibetwindow import QuodLibetWindow
+    window = QuodLibetWindow(library, player)
+    widgets.main = window
+
+    events = EventPlugins(library.librarian, player, [
+        os.path.join(const.BASEDIR, "plugins", "events"),
+        os.path.join(const.USERDIR, "plugins", "events")], "events")
+    events.rescan()
+
+    for p in [playorder, SongsMenu.plugins, SongProperties.plugins, events]:
+        window.connect('destroy', p.destroy)
+
+    return window
 
 def print_fifo(command):
     if not os.path.exists(const.CURRENT):
@@ -97,29 +175,28 @@ def print_playing(fstring="<artist~album~tracknumber~title>"):
     from quodlibet.parse import Pattern
 
     try:
-        fn = file(const.CURRENT)
-        data = {}
-        for line in fn:
-            line = line.strip()
-            parts = line.split("=")
-            key = parts[0]
-            val = "=".join(parts[1:])
-            if key.startswith("~#"):
-                try: data[key] = int(val)
-                except ValueError:
-                    try: data[key] = float(val)
-                    except ValueError: data[key] = 0
-            else:
-                if key != "~filename": val = util.decode(val)
-                else: val = util.decode(util.fsencode(val))
-                if key in data: data[key] += "\n" + val
-                else: data[key] = val
-        print_(Pattern(fstring).format(AudioFile(data)))
+        text = open(const.CURRENT, "rb").read()
+        song = AudioFile()
+        song.from_dump(text)
+        print_(Pattern(fstring).format(song))
         raise SystemExit
     except (OSError, IOError):
         print_(_("No song is currently playing."))
         raise SystemExit(True)
 
+def print_query(query):
+    '''Queries library, dumping filenames of matches to stdout
+
+        See Issue 716
+    '''
+    print_d("Querying library for %r" %query)
+    import quodlibet.library
+    library = quodlibet.library.init(const.LIBRARY)
+    songs = library.query(query)
+    #songs.sort()
+    sys.stdout.write("\n".join([song("~filename") for song in songs]) + "\n")
+    raise SystemExit
+
 def isrunning():
     return os.path.exists(const.CONTROL)
 
@@ -144,28 +221,10 @@ def control(c):
         else:
             raise SystemExit
 
-def enable_periodic_save(library):
-    # Check every 5 minutes to see if the library/config on disk are
-    # over 15 minutes old; if so, update them. This function can, in theory,
-    # break if saving the library takes more than 5 minutes.
-    import gobject
-
-    def save():
-        if library.dirty:
-            if (time.time() - util.mtime(const.LIBRARY)) > 15*60:
-                library.save(const.LIBRARY)
-            if (time.time() - util.mtime(const.CONFIG)) > 15*60:
-                config.write(const.CONFIG)
-        thread = Thread(target=save)
-        gobject.timeout_add(
-            5*60000, thread.start, priority=gobject.PRIORITY_LOW)
-    thread = Thread(target=save)
-    gobject.timeout_add(5*60000, thread.start, priority=gobject.PRIORITY_LOW)
-
 def process_arguments():
     controls = ["next", "previous", "play", "pause", "play-pause",
                 "hide-window", "show-window", "toggle-window",
-                "focus", "quit", "unfilter", "refresh"]
+                "focus", "quit", "unfilter", "refresh", "force-previous"]
     controls_opt = ["seek", "order", "repeat", "query", "volume", "filter",
                     "set-rating", "set-browser", "open-browser", "random",
                     "song-list", "queue"]
@@ -173,14 +232,16 @@ def process_arguments():
     options = util.OptionParser(
         "Quod Libet", const.VERSION,
         _("a music library and player"),
-        _("[ --print-playing | control ]"))
+        _("[option]"))
 
     options.add("print-playing", help=_("Print the playing song and exit"))
     options.add("start-playing", help=_("Begin playing immediately"))
 
     for opt, help in [
         ("next", _("Jump to next song")),
-        ("previous", _("Jump to previous song")),
+        ("previous",
+            _("Jump to previous song or restart if near the beginning")),
+        ("force-previous", _("Jump to previous song")),
         ("play", _("Start playback")),
         ("pause", _("Pause playback")),
         ("play-pause", _("Toggle play/pause mode")),
@@ -201,7 +262,7 @@ def process_arguments():
     for opt, help, arg in [
         ("seek", _("Seek within the playing song"), _("[+|-][HH:]MM:SS")),
         ("order", _("Set or toggle the playback order"),
-         "[order]|toggle"),
+            "[order]|toggle"),
         ("repeat", _("Turn repeat off, on, or toggle it"), "0|1|t"),
         ("volume", _("Set the volume"), "(+|-|)0..100"),
         ("query", _("Search your audio library"), _("query")),
@@ -213,10 +274,14 @@ def process_arguments():
         ("song-list", _("Show or hide the main song list"), "on|off|t"),
         ("random", _("Filter on a random value"), Q_("command|tag")),
         ("filter", _("Filter on a tag value"), _("tag=value")),
-        ("enqueue", _("Enqueue a file or query"), "%s|%s" %(
-        Q_("command|filename"), _("query"))),
-        ("unqueue", _("Unqueue a file or query"), "%s|%s" %(
-        Q_("command|filename"), _("query"))),
+        ("enqueue", _("Enqueue a file or query"), "%s|%s" % (
+            Q_("command|filename"), _("query"))),
+        ("enqueue-files", _("Enqueue comma-separated files"), "%s[,%s..]" % (
+            _("filename"), _("filename"))),
+        ("print-query", _("Print filenames of results of query to stdout"), 
+            _("query")),
+        ("unqueue", _("Unqueue a file or query"), "%s|%s" % (
+            Q_("command|filename"), _("query"))),
         ]: options.add(opt, help=help, arg=arg)
 
     options.add("sm-config-prefix", arg="dummy")
@@ -249,7 +314,6 @@ def process_arguments():
         }
 
     opts, args = options.parse()
-
     for command, arg in opts.items():
         if command in controls: control(command)
         elif command in controls_opt:
@@ -269,6 +333,8 @@ def process_arguments():
             except ValueError:
                 filename = arg
             control(command + " " + filename)
+        elif command == "enqueue-files":
+            control(command + " " + arg)
         elif command == "play-file":
             try:
                 filename = URI(arg).filename
@@ -279,13 +345,17 @@ def process_arguments():
         elif command == "print-playing":
             try: print_playing(args[0])
             except IndexError: print_playing()
+        elif command == "print-query":
+            print_query(arg)
         elif command == "start-playing":
             global play
             play = True
 
 if __name__ == "__main__":
+    quodlibet._init_signal()
+
     process_arguments()
-    if isrunning() and not os.environ.get("QUODLIBET_DEBUG"):
+    if isrunning() and not quodlibet.const.DEBUG:
         print_(_("Quod Libet is already running."))
         control('focus')
     main()
