@@ -75,9 +75,20 @@ class Browser(object):
     def init(klass, library): pass
     init = classmethod(init)
 
-    # Returns true if the song should remain on the song list. Used to
-    # implement dynamic playlist removal when a song ends.
-    def dynamic(self, song): return True
+    # Deprecated: use active_filter instead
+    def dynamic(self, song):
+        if callable(self.active_filter):
+            ret = self.active_filter(song)
+            if ret is not None:
+                return ret
+        return True
+
+    # A callable that returns True if the passed song should be in the
+    # song list, False if not and None if no filter is active.
+    # Used for adding new songs to the song list or
+    # dynamic playlist removal when a song ends.
+    # def active_filter(self, song): ...
+    active_filter = None
 
     # Save/restore selected songlist. Browsers should save whatever
     # they need to recreate the criteria for the current song list (not
@@ -130,7 +141,7 @@ class Browser(object):
         return ngettext(
             "%(count)d song (%(time)s)", "%(count)d songs (%(time)s)", i)
 
-    # Return a list of values for the given tag. This needs to be
+    # Return a list of unique values for the given tag. This needs to be
     # here since not all browsers pull from the default library.
     def list(self, tag):
         library = quodlibet.library.library
