@@ -16,10 +16,8 @@ from quodlibet import config
 from quodlibet import const
 from quodlibet import qltk
 from quodlibet import util
-from quodlibet import stock
 
 from quodlibet.browsers._base import Browser
-from quodlibet.browsers.search import BoxSearchBar
 from quodlibet.parse import Query, XMLFromPattern
 from quodlibet.qltk.ccb import ConfigCheckButton
 from quodlibet.qltk.completion import EntryWordCompletion
@@ -27,7 +25,9 @@ from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.textedit import PatternEditBox
 from quodlibet.qltk.views import AllTreeView
 from quodlibet.qltk.x import MenuItem
-from quodlibet.util import copool, gobject_weak
+from quodlibet.qltk.searchbar import SearchBarBox
+from quodlibet.qltk.menubutton import MenuButton
+from quodlibet.util import copool, gobject_weak, thumbnails
 from quodlibet.util.library import background_filter
 
 EMPTY = _("Songs not in an album")
@@ -81,21 +81,22 @@ class Preferences(qltk.UniqueWindow):
         super(Preferences, self).__init__()
         self.set_border_width(12)
         self.set_title(_("Album List Preferences") + " - Quod Libet")
-        self.add(gtk.VBox(spacing=6))
-        self.set_default_size(300, 250)
+        self.set_default_size(400, 270)
         self.set_transient_for(qltk.get_top_parent(parent))
 
+        box = gtk.VBox(spacing=6)
+
         cb = ConfigCheckButton(
             _("Show album _covers"), "browsers", "album_covers")
         cb.set_active(config.getboolean("browsers", "album_covers"))
         gobject_weak(cb.connect, 'toggled', lambda s: AlbumList.toggle_covers())
-        self.child.pack_start(cb, expand=False)
+        box.pack_start(cb, expand=False)
 
         cb = ConfigCheckButton(
             _("Inline _search includes people"),
             "browsers", "album_substrings")
         cb.set_active(config.getboolean("browsers", "album_substrings"))
-        self.child.pack_start(cb, expand=False)
+        box.pack_start(cb, expand=False)
 
         vbox = gtk.VBox(spacing=6)
         label = gtk.Label()
@@ -110,8 +111,20 @@ class Preferences(qltk.UniqueWindow):
         vbox.pack_start(edit)
         self.__preview_pattern(edit, label)
         f = qltk.Frame(_("Album Display"), child=vbox)
-        self.child.pack_start(f)
+        box.pack_start(f)
 
+        main_box = gtk.VBox(spacing=12)
+        close = gtk.Button(stock=gtk.STOCK_CLOSE)
+        close.connect('clicked', lambda *x: self.destroy())
+        b = gtk.HButtonBox()
+        b.set_layout(gtk.BUTTONBOX_END)
+        b.pack_start(close)
+
+        main_box.pack_start(box)
+        main_box.pack_start(b, expand=False)
+        self.add(main_box)
+
+        close.grab_focus()
         self.show_all()
 
     def __set_pattern(self, apply, edit):
@@ -138,7 +151,221 @@ class Preferences(qltk.UniqueWindow):
         else: edit.apply.set_sensitive(True)
         label.set_markup(text)
 
-class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
+def cmpa(a, b):
+    """Like cmp but treats values that evaluate to false as inf"""
+    if not a and b: return 1
+    if not b and a: return -1
+    return cmp(a, b)
+
+class PreferencesButton(gtk.HBox):
+    def __init__(self, model):
+        super(PreferencesButton, self).__init__()
+
+        sort_orders = [
+            (_("_Title"), self.__compare_title),
+            (_("_Artist"), self.__compare_artist),
+            (_("_Date"), self.__compare_date),
+            (_("_Genre"), self.__compare_genre),
+            ]
+
+        menu = gtk.Menu()
+
+        sort_item = gtk.MenuItem(_("Sort _by..."))
+        sort_menu = gtk.Menu()
+
+        active = config.getint('browsers', 'album_sort', 1)
+
+        item = None
+        for i, (label, func) in enumerate(sort_orders):
+            item = gtk.RadioMenuItem(item, label)
+            model.set_sort_func(100 + i, func)
+            if i == active:
+                model.set_sort_column_id(100 + i, gtk.SORT_ASCENDING)
+                item.set_active(True)
+            gobject_weak(item.connect, "toggled",
+                         util.DeferredSignal(self.__sort_toggled_cb),
+                         model, i)
+            sort_menu.append(item)
+
+        sort_item.set_submenu(sort_menu)
+        menu.append(sort_item)
+
+        pref_item = MenuItem(_("_Preferences"), gtk.STOCK_PREFERENCES)
+        menu.append(pref_item)
+        gobject_weak(pref_item.connect, "activate", Preferences)
+
+        menu.show_all()
+
+        button = MenuButton(
+            gtk.image_new_from_stock(
+                gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_MENU),
+                arrow=False)
+        button.set_menu(menu)
+        self.pack_start(button)
+
+    def __sort_toggled_cb(self, item, model, num):
+        if item.get_active():
+            config.set("browsers", "album_sort", str(num))
+            model.set_sort_column_id(100 + num, gtk.SORT_ASCENDING)
+
+    def __compare_title(self, model, i1, i2):
+        a1, a2 = model[i1][0], model[i2][0]
+        # all albums has to stay at the top
+        if (a1 and a2) is None: return cmp(a1, a2)
+        # move album without a title to the bottom
+        if not a1.title: return 1
+        if not a2.title: return -1
+        return (cmpa(a1.sort, a2.sort) or
+                cmp(a1.key, a2.key))
+
+    def __compare_artist(self, model, i1, i2):
+        a1, a2 = model[i1][0], model[i2][0]
+        if (a1 and a2) is None: return cmp(a1, a2)
+        if not a1.title: return 1
+        if not a2.title: return -1
+        return (cmpa(a1.peoplesort, a2.peoplesort) or
+                cmpa(a1.date, a2.date) or
+                cmpa(a1.sort, a2.sort) or
+                cmp(a1.key, a2.key))
+
+    def __compare_date(self, model, i1, i2):
+        a1, a2 = model[i1][0], model[i2][0]
+        if (a1 and a2) is None: return cmp(a1, a2)
+        if not a1.title: return 1
+        if not a2.title: return -1
+        return (cmpa(a1.date, a2.date) or
+                cmpa(a1.sort, a2.sort) or
+                cmp(a1.key, a2.key))
+
+    def __compare_genre(self, model, i1, i2):
+        a1, a2 = model[i1][0], model[i2][0]
+        if (a1 and a2) is None: return cmp(a1, a2)
+        if not a1.title: return 1
+        if not a2.title: return -1
+        return (cmpa(a1.genre, a2.genre) or
+                cmpa(a1.peoplesort, a2.peoplesort) or
+                cmpa(a1.date, a2.date) or
+                cmpa(a1.sort, a2.sort) or
+                cmp(a1.key, a2.key))
+
+
+class VisibleUpdate(object):
+
+    # how many rows should be updated
+    # beyond the visible area in both directions
+    PRELOAD_COUNT = 35
+
+    def enable_row_update(self, view, sw, column):
+        gobject_weak(view.connect_object, 'expose-event',
+                     self.__update_visibility, view)
+
+        gobject_weak(sw.get_vadjustment().connect, "value-changed",
+                     self.__stop_update ,view)
+
+        self.__pending_paths = []
+        self.__scan_timeout = None
+        self.__column = column
+        self.__first_expose = True
+
+    def disable_row_update(self):
+        if self.__scan_timeout:
+            gobject.source_remove(self.__scan_timeout)
+            self.__scan_timeout = None
+
+        if self.__pending_paths:
+            copool.remove(self.__scan_paths)
+
+        self.__column = None
+        self.__pending_paths = []
+
+    def _row_needs_update(self, row):
+        """Should return True if the rows should be updated"""
+        raise NotImplementedError
+
+    def _update_row(self, row):
+        """Do whatever is needed to update the row"""
+        raise NotImplementedError
+
+    def __stop_update(self, adj, view):
+        if self.__pending_paths:
+            copool.remove(self.__scan_paths)
+            self.__pending_paths = []
+            self.__update_visibility(view)
+
+    def __update_visibility(self, view, *args):
+        if not self.__column.get_visible():
+            return
+
+        # update all visible rows on first expose event
+        if self.__first_expose:
+            self.__first_expose = False
+            self.__update_visible_rows(view, 0)
+            for i in self.__scan_paths(): pass
+
+        if self.__scan_timeout:
+            gobject.source_remove(self.__scan_timeout)
+            self.__scan_timeout = None
+
+        self.__scan_timeout = gobject.timeout_add(
+            50, self.__update_visible_rows, view, self.PRELOAD_COUNT)
+
+    def __scan_paths(self):
+        while self.__pending_paths:
+            model, path = self.__pending_paths.pop()
+            try:
+                row = model[path]
+            # row could have gone away by now
+            except IndexError: pass
+            else:
+                self._update_row(row)
+                yield True
+
+    def __update_visible_rows(self, view, preload):
+        vrange = view.get_visible_range()
+        if vrange is None: return
+
+        model_filter = view.get_model()
+        model = model_filter.get_model()
+
+        #generate a path list so that cover scanning starts in the middle
+        #of the visible area and alternately moves up and down
+        start, end = vrange
+
+        # pygtk2.12 sometimes returns empty tuples
+        if not start or not end:
+            return
+
+        start = start[0] - preload - 1
+        end = end[0] + preload
+
+        vlist = range(end, start, -1)
+        top = vlist[:len(vlist)/2]
+        bottom = vlist[len(vlist)/2:]
+        top.reverse()
+
+        vlist_new = []
+        for i in vlist:
+            if top: vlist_new.append(top.pop())
+            if bottom: vlist_new.append(bottom.pop())
+        vlist_new = filter(lambda s: s >= 0, vlist_new)
+
+        visible_paths = []
+        for path in vlist_new:
+            model_path = model_filter.convert_path_to_child_path(path)
+            try:
+                row = model[model_path]
+            except TypeError:
+                pass
+            else:
+                if self._row_needs_update(row):
+                    visible_paths.append([model, model_path])
+
+        if not self.__pending_paths and visible_paths:
+            copool.add(self.__scan_paths)
+        self.__pending_paths = visible_paths
+
+
+class AlbumList(Browser, gtk.VBox, util.InstanceTracker, VisibleUpdate):
     expand = qltk.RHPaned
     __gsignals__ = Browser.__gsignals__
     __model = None
@@ -162,13 +389,14 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         except EnvironmentError:
             klass._pattern_text = PATTERN
 
-        no_cover = os.path.join(const.IMAGEDIR, stock.NO_COVER)
+        theme = gtk.icon_theme_get_default()
         try:
-            klass.__no_cover = gtk.gdk.pixbuf_new_from_file_at_size(
-                no_cover + ".svg", 48, 48)
-        except gobject.GError:
-            klass.__no_cover = gtk.gdk.pixbuf_new_from_file_at_size(
-                no_cover + ".png", 48, 48)
+            klass.__no_cover = theme.load_icon(
+                "quodlibet-missing-cover", 48, 0)
+        except gobject.GError: pass
+        else:
+            klass.__no_cover = thumbnails.scale(
+                klass.__no_cover, (48, 48))
 
         klass._pattern = XMLFromPattern(klass._pattern_text)
 
@@ -244,88 +472,11 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
                 model.row_changed(row.path, row.iter)
                 if not changed_albums: break
 
-    class FilterBar(BoxSearchBar):
-        """The search filter entry HBox, modifiedto toggle between the search
-        bar and album list on mnemonic activation."""
-        def __init__(self, albumlist, *args, **kwargs):
-            super(AlbumList.FilterBar, self).__init__(*args, **kwargs)
-            self.albumlist = albumlist
-
-        def _mnemonic_activate(self, label, group_cycling):
-            widget = label.get_mnemonic_widget()
-            if widget.is_focus():
-                self.albumlist.view.grab_focus()
-                return True
-
-    class SortCombo(gtk.ComboBox):
-        """ComboBox which sets the sort function on a TreeModelSort."""
-        def __init__(self, model):
-            # Contains string to display, function to do sorting
-            cbmodel = gtk.ListStore(str)
-            gtk.ComboBox.__init__(self, cbmodel)
-            cell = gtk.CellRendererText()
-            self.pack_start(cell, True)
-            self.add_attribute(cell, 'text', 0)
-            model.set_sort_func(100, self.__compare_title)
-            model.set_sort_func(101, self.__compare_artist)
-            model.set_sort_func(102, self.__compare_date)
-
-            for text in [_("Title"), _("Artist"), _("Date")]:
-                cbmodel.append(row=[text])
-
-            gobject_weak(self.connect_object, 'changed',
-                self.__set_cmp_func, model)
-
-            try: active = config.getint('browsers', 'album_sort')
-            except: active = 0
-            self.set_active(active)
-
-        def __set_cmp_func(self, model):
-            active = self.get_active()
-            config.set("browsers", "album_sort", str(active))
-            model.set_sort_column_id(100 + active, gtk.SORT_ASCENDING)
-
-        def __compare_title(self, model, i1, i2):
-            a1, a2 = model[i1][0], model[i2][0]
-            if (a1 and a2) is None: return cmp(a1, a2)
-            elif not a1.title: return 1
-            elif not a2.title: return -1
-            elif not a1.sort: return 1
-            elif not a2.sort: return -1
-            else: return cmp((a1.sort, a1.key), (a2.sort, a2.key))
-
-        def __compare_artist(self, model, i1, i2):
-            a1, a2 = model[i1][0], model[i2][0]
-            if (a1 and a2) is None: return cmp(a1, a2)
-            elif not a1.title: return 1
-            elif not a2.title: return -1
-            elif not a1.sort: return 1
-            elif not a2.sort: return -1
-            elif not a1.peoplesort and a2.peoplesort: return 1
-            elif not a2.peoplesort and a1.peoplesort: return -1
-            else: return (cmp(a1.peoplesort and a1.peoplesort[0],
-                              a2.peoplesort and a2.peoplesort[0]) or
-                          cmp(a1.date or "ZZZZ", a2.date or "ZZZZ") or
-                          cmp((a1.sort, a1.key), (a2.sort, a2.key)))
-
-        def __compare_date(self, model, i1, i2):
-            a1, a2 = model[i1][0], model[i2][0]
-            if (a1 and a2) is None: return cmp(a1, a2)
-            elif not a1.title: return 1
-            elif not a2.title: return -1
-            elif not a1.sort: return 1
-            elif not a2.sort: return -1
-            elif not a1.date and a2.date: return 1
-            elif not a2.date and a1.date: return -1
-            else: return (cmp(a1.date, a2.date) or
-                cmp((a1.sort, a1.key), (a2.sort, a2.key)))
-
     def __init__(self, library, player):
         super(AlbumList, self).__init__(spacing=6)
         self._register_instance()
         if self.__model is None:
             self._init_model(library)
-        self.__save = bool(player)
 
         sw = gtk.ScrolledWindow()
         sw.set_shadow_type(gtk.SHADOW_IN)
@@ -338,14 +489,6 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         self.__filter = None
         model_filter.set_visible_func(self.__parse_query)
 
-        self.__pending_covers = []
-        self.__scan_timeout = None
-        gobject_weak(view.connect_object, 'expose-event',
-            self.__update_visibility, view)
-
-        gobject_weak(sw.get_vadjustment().connect, "value-changed",
-            self.__stop_cover_update)
-
         render = gtk.CellRendererPixbuf()
         self.__cover_column = column = gtk.TreeViewColumn("covers", render)
         column.set_visible(config.getboolean("browsers", "album_covers"))
@@ -400,7 +543,7 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
                 self.__play_selection, player)
 
         self.__sig = gobject_weak(view.get_selection().connect, 'changed',
-            self.__selection_changed, parent=view)
+            util.DeferredSignal(self.__update_songs), parent=view)
 
         targets = [("text/x-quodlibet-songs", gtk.TARGET_SAME_APP, 1),
                    ("text/uri-list", 0, 2)]
@@ -410,32 +553,36 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         gobject_weak(view.connect_object, 'popup-menu',
             self.__popup, view, library)
 
-        search = AlbumList.FilterBar(
-                self, library, button=False, completion=AlbumTagCompletion())
-        search.callback = self.__update_filter
-        prefs = gtk.Button()
-        prefs.add(gtk.image_new_from_stock(
-            gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_MENU))
-        gobject_weak(prefs.connect, 'clicked', Preferences)
+        self.accelerators = gtk.AccelGroup()
+        search = SearchBarBox(button=False, completion=AlbumTagCompletion(),
+                              accel_group=self.accelerators)
+        gobject_weak(search.connect, 'query-changed', self.__update_filter)
+        gobject_weak(search.connect_object,
+                     'focus-out', lambda w: w.grab_focus(), view)
+        self.__search = search
+
+        prefs = PreferencesButton(model_sort)
         search.pack_start(prefs, expand=False)
         self.pack_start(search, expand=False)
         self.pack_start(sw, expand=True)
 
-        hb = gtk.HBox(spacing=6)
-        l = gtk.Label(_("Sort _by:"))
-        l.set_use_underline(True)
-        sc = self.SortCombo(model_sort)
-        l.set_mnemonic_widget(sc)
-        hb.pack_start(l, expand=False)
-        hb.pack_start(sc)
-        self.pack_start(hb, expand=False)
-
         self.connect("destroy", self.__destroy)
 
+        self.enable_row_update(view, sw, self.__cover_column)
+
         self.show_all()
 
+    def _row_needs_update(self, row):
+        album = row[0]
+        return album is not None and not album.scanned
+
+    def _update_row(self, row):
+        album = row[0]
+        album.scan_cover()
+        self._refresh_albums([album])
+
     def __destroy(self, browser):
-        copool.remove(self.__scan_covers)
+        self.disable_row_update()
 
         # https://bugzilla.gnome.org/show_bug.cgi?id=624112
         # filter model keeps its filter function reference.
@@ -452,92 +599,27 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         if not klass.instances():
             klass._destroy_model()
 
-    def __update_visible_covers(self, view):
-        vrange = view.get_visible_range()
-        if vrange is None: return
-
-        model_filter = view.get_model()
-        model = model_filter.get_model()
-
-        #generate a path list so that cover scanning starts in the middle
-        #of the visible area and alternately moves up and down
-        preload_count = 35
-
-        start, end = vrange
-        start = start[0] - preload_count - 1
-        end = end[0] + preload_count
-
-        vlist = range(end, start, -1)
-        top = vlist[:len(vlist)/2]
-        bottom = vlist[len(vlist)/2:]
-        top.reverse()
-
-        vlist_new = []
-        for i in vlist:
-            if top: vlist_new.append(top.pop())
-            if bottom: vlist_new.append(bottom.pop())
-        vlist_new = filter(lambda s: s >= 0, vlist_new)
-
-        visible_albums = []
-        for path in vlist_new:
-            model_path = model_filter.convert_path_to_child_path(path)
-            try:
-                row = model[model_path]
-            except TypeError:
-                pass
-            else:
-                album = row[0]
-                if album is not None and not album.scanned:
-                    visible_albums.append([model, model_path])
-
-        if not self.__pending_covers and visible_albums:
-            copool.add(self.__scan_covers)
-        self.__pending_covers = visible_albums
-
-    def __scan_covers(self):
-        while self.__pending_covers:
-            model, path = self.__pending_covers.pop()
-            try:
-                row = model[path]
-            # row could have gone away by now
-            except IndexError: pass
-            else:
-                album = row[0]
-                album.scan_cover()
-                self._refresh_albums([album])
-            yield True
-
-    def __stop_cover_update(self, *args):
-        self.__pending_covers = []
-
-    def __update_visibility(self, view, *args):
-        if not self.__cover_column.get_visible():
-            return
-
-        if self.__scan_timeout:
-            gobject.source_remove(self.__scan_timeout)
-            self.__scan_timeout = None
-
-        self.__scan_timeout = gobject.timeout_add(
-            50, self.__update_visible_covers, view)
-
-    def __update_filter(self, text):
-        #This could be called after the browsers is already closed
-        if not self.view.get_selection(): return
+    def __update_filter(self, entry, text, restore=False):
         model = self.view.get_model()
-        if Query.is_parsable(text):
-            if not text:
-                self.__filter = None
-            else:
-                self.__filter = Query(text, star=["~people", "album"]).search
+
+        self.__filter = None
+        if not Query.match_all(text):
+            self.__filter = Query(text, star=["~people", "album"]).search
         self.__bg_filter = background_filter()
+
         self.__inhibit()
+
         # We could be smart and try to scroll to a selected album
         # but that introduces lots of wild scrolling. Feel free to change it.
         # Without scrolling the TV trys to stay at the same position (40% down)
         # which makes no sence so always go to the top.
-        self.view.scroll_to_point(0, 0)
-        model.refilter()
+        if not restore:
+            self.view.scroll_to_point(0, 0)
+
+        # don't filter on restore if there is nothing to filter
+        if not restore or self.__filter or self.__bg_filter:
+            model.refilter()
+
         self.__uninhibit()
 
     def __parse_query(self, model, iter):
@@ -627,27 +709,48 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
     def __play_selection(self, view, indices, col, player):
         player.reset()
 
+    def active_filter(self, song):
+        selection = self.view.get_selection()
+        if not selection:
+            return
+
+        model, rows = selection.get_selected_rows()
+        if not model or not rows:
+            return
+
+        if rows and model[rows[0]][0] is None:
+            return True
+
+        key = song.album_key
+        for album in [model[row][0] for row in rows]:
+            if key == album.key:
+                return True
+
+        return False
+
     def filter(self, key, values):
         assert(key == "album")
         if not values: values = [""]
         view = self.view
+        self.__inhibit()
         selection = view.get_selection()
-        selection.unselect_all()
         model = view.get_model()
-        first = None
+        first = True
         for row in model:
             if row[0] is not None and row[0].title in values:
-                selection.select_path(row.path)
-                if first is None:
+                if first:
+                    view.scroll_to_cell(row.path[0],
+                        use_align=True, row_align=0.5)
                     view.set_cursor(row.path)
-                    first = row.path[0]
-        if first:
-            view.scroll_to_cell(first, use_align=True, row_align=0.5)
+                    first = False
+                else:
+                    selection.select_path(row.path)
+        self.__uninhibit()
+        if not first:
+            selection.emit('changed')
 
     def unfilter(self):
-        selection = self.view.get_selection()
-        selection.unselect_all()
-        selection.select_path((0,))
+        self.view.set_cursor((0,))
 
     def activate(self):
         self.view.get_selection().emit('changed')
@@ -667,6 +770,14 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         self.view.get_selection().handler_unblock(self.__sig)
 
     def restore(self):
+        text = config.get("browsers", "query_text").decode("utf-8")
+        entry = self.__search
+        entry.set_text(text)
+
+        # update_filter expects a parsable query
+        if Query.is_parsable(text):
+            self.__update_filter(entry, text, restore=True)
+
         albums = config.get("browsers", "albums").split("\n")
         view = self.view
         selection = view.get_selection()
@@ -674,18 +785,21 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         # no albums. If it's "" and some other stuff, assume no albums,
         # otherwise all albums.
         self.__inhibit()
-        selection.unselect_all()
-        if albums == [""]:  selection.select_path((0,))
+        if albums == [""]:
+            selection.unselect_all()
+            selection.select_path((0,))
         else:
             model = selection.get_tree_view().get_model()
-            first = None
+            first = True
             for row in model:
                 if row[0] is not None and row[0].title in albums:
-                    selection.select_path(row.path)
-                    first = first or row.path
-
-            if first:
-                view.scroll_to_cell(first[0], use_align=True, row_align=0.5)
+                    if first:
+                        view.scroll_to_cell(row.path, use_align=True,
+                                            row_align=0.5)
+                        view.set_cursor(row.path)
+                        first = False
+                    else:
+                        selection.select_path(row.path)
         self.__uninhibit()
 
     def scroll(self, song):
@@ -695,17 +809,9 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
         for row in model:
             if row[0] is not None and row[0].key == album_key:
                 view.scroll_to_cell(row.path[0], use_align=True, row_align=0.5)
-                sel = view.get_selection()
-                sel.unselect_all()
-                sel.select_path(row.path[0])
+                view.set_cursor(row.path)
                 break
 
-    def __selection_changed(self, selection):
-        # Without this delay, GTK+ seems to sometimes call this function
-        # before model elements are totally filled in, leading to errors
-        # like "TypeError: unknown type (null)".
-        gobject.idle_add(self.__update_songs, selection.get_tree_view())
-
     def __get_config_string(self, selection):
         if not selection: return ""
         model, rows = selection.get_selected_rows()
@@ -723,13 +829,16 @@ class AlbumList(Browser, gtk.VBox, util.InstanceTracker):
             confval = "\n" + confval[:-1]
         return confval
 
-    def __update_songs(self, view):
+    def save(self):
+        selection = self.view.get_selection()
+        conf = self.__get_config_string(selection)
+        config.set("browsers", "albums", conf)
+        text = self.__search.get_text().encode("utf-8")
+        config.set("browsers", "query_text", text)
+
+    def __update_songs(self, selection):
         if not self.__dict__: return
-        selection = view.get_selection()
         songs = self.__get_selected_songs(selection, False)
         self.emit('songs-selected', songs, None)
-        if self.__save:
-            conf = self.__get_config_string(selection)
-            config.set("browsers", "albums", conf)
 
 browsers = [AlbumList]
