@@ -55,28 +55,7 @@ class FileSystem(Browser, gtk.ScrolledWindow):
         self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         self.set_shadow_type(gtk.SHADOW_IN)
 
-        def get_win32_drives():
-            import string
-            from ctypes import windll
-            drives = []
-            bitmask = windll.kernel32.GetLogicalDrives()
-            for letter in string.uppercase:
-                if bitmask & 1:
-                    drives.append(letter + ":\\")
-                bitmask >>= 1
-            return drives
-
         folders = filter(None, split_scan_dirs(config.get("settings", "scan")))
-        if folders:
-            folders.append(None)
-        if const.HOME not in folders:
-            folders.append(const.HOME)
-        if sys.platform == "win32":
-            for folder in get_win32_drives():
-                if folder not in folders:
-                    folders.append(folder)
-        elif "/" not in folders:
-            folders.append("/")
 
         dt = DirectoryTree(folders=folders)
         targets = [("text/x-quodlibet-songs", gtk.TARGET_SAME_APP, 1),
@@ -87,8 +66,8 @@ class FileSystem(Browser, gtk.ScrolledWindow):
         sel = dt.get_selection()
         sel.unselect_all()
         sel.connect_object('changed', copool.add, self.__songs_selected, dt)
-        if player: dt.connect('row-activated', self.__play, player)
-        else: self.save = lambda: None
+        if player:
+            dt.connect('row-activated', lambda *a: player.reset())
         self.add(dt)
         self.show_all()
 
@@ -112,9 +91,6 @@ class FileSystem(Browser, gtk.ScrolledWindow):
             uris = [song("~uri") for song in songs]
             sel.set_uris(uris)
 
-    def __play(self, view, indices, column, player):
-        player.reset()
-
     def can_filter(self, key):
         return (key == "~dirname")
 
@@ -138,14 +114,15 @@ class FileSystem(Browser, gtk.ScrolledWindow):
             if model[iter][0] in paths:
                 self.child.get_selection().select_path(path)
                 paths.remove(model[iter][0])
-                if not first: first.append(path)
+                if not first:
+                    self.child.set_cursor(path)
+                    first.append(path)
             else:
                 for fpath in paths:
                     if model[path][0] and fpath.startswith(model[path][0]):
                         self.child.expand_row(path, False)
             return not bool(paths)
         first = []
-        self.child.get_selection().unselect_all()
         self.child.get_model().foreach(select, (paths, first))
         if first: self.child.scroll_to_cell(first[0], None, True, 0.5)
 
@@ -216,6 +193,5 @@ class FileSystem(Browser, gtk.ScrolledWindow):
         if self.window:
             self.window.set_cursor(None)
         self.emit('songs-selected', songs, None)
-        self.save()
 
 browsers = [FileSystem]
