@@ -1,4 +1,4 @@
-# Copyright 2004 Joe Wreschnig
+# Copyright 2004-2011 Joe Wreschnig, Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -14,13 +14,44 @@ import const
 from ConfigParser import RawConfigParser as ConfigParser, Error as error
 
 _config = ConfigParser()
-get = _config.get
-set = _config.set
-getboolean = _config.getboolean
-getint = _config.getint
-getfloat = _config.getfloat
 options = _config.options
 
+def get(*args):
+    if len(args) == 3:
+        try: return _config.get(*args[:2])
+        except error: return args[-1]
+    return _config.get(*args)
+
+def getboolean(*args):
+    if len(args) == 3:
+        if not isinstance(args[-1], bool): raise ValueError
+        try: return _config.getboolean(*args[:2])
+        # ValueError if the value found in the config file
+        # does not match any string representation -> so catch it too
+        except (ValueError, error): return args[-1]
+    return _config.getboolean(*args)
+
+def getint(*args):
+    if len(args) == 3:
+        if not isinstance(args[-1], int): raise ValueError
+        try: return _config.getint(*args[:2])
+        except error: return args[-1]
+    return _config.getint(*args)
+
+def getfloat(*args):
+    if len(args) == 3:
+        if not isinstance(args[-1], float): raise ValueError
+        try: return _config.getfloat(*args[:2])
+        except error: return args[-1]
+    return _config.getfloat(*args)
+
+# RawConfigParser only allows string values but doesn't scream if they are not
+# (and it only fails before the first config save..)
+def set(section, option, value):
+    if not isinstance(value, str):
+        value = str(value)
+    _config.set(section, option, value)
+
 def write(filename):
     if isinstance(filename, basestring):
         if not os.path.isdir(os.path.dirname(filename)):
@@ -30,6 +61,12 @@ def write(filename):
     _config.write(f)
     f.close()
 
+def save(filename):
+    print_d("Writing config...")
+    try: write(filename)
+    except EnvironmentError:
+        print_w("Unable to write config.")
+
 def quit():
     for section in _config.sections():
         _config.remove_section(section)
@@ -48,6 +85,7 @@ def init(*rc_files):
           "pre_amp_gain" : "0.0",
           "backend": "gstbe",
           "gst_pipeline": "",
+          "gst_buffer": "1.5", # stream buffer duration in seconds
           },
 
         "library":
@@ -66,7 +104,7 @@ def init(*rc_files):
           "song": "", # filename of last song
           "seek": "0", # last song position, in milliseconds
           "volume": "1.0", # internal volume, [0.0, 1.0]
-          "browser": "SearchBar", # browser name
+          "browser": "PanedBrowser", # browser name
           "songlist": "true", # on or off
           "queue": "false", # on or off
           "shufflequeue": "false", # on or off
@@ -77,13 +115,14 @@ def init(*rc_files):
         "browsers":
         { "query_text": "", # none/search bar text
           "color": "true", # color search terms in search bar
-          "panes": "~people	<~year|<~year> - <album>|<album>>", # panes in paned browser
+          "panes": "~people	<~year|\<b\>\<i\><~year>\</i\>\</b\> - ><album>", # panes in paned browser
           "pane_selection": "", # selected pane values
           "background": "", # "global" filter for SearchBar
           "albums": "", # album list
           "album_sort": "0", # album sorting mode, default is title
           "album_covers": "1", # album cover display, on/off
           "album_substrings": "1", # include substrings in inline search
+          "radio": "", # radio filter selection
           "rating_click": "true", # click to rate song, on/off
           "rating_confirm_multiple": "false", # confirm rating multiple songs
           },
@@ -91,8 +130,7 @@ def init(*rc_files):
         # Kind of a dumping ground right now, should probably be
         # cleaned out later.
         "settings":
-        { "scan": "", # scan directories, :-separatd
-          "round": "true", # use rounded corners for artwork thumbnails
+        { "scan": "", # scan directories, :-separated
           "jump": "true", # scroll song list on current song change
           "default_rating": "0.5", # initial rating of new song
           "ratings": "4", # maximum rating value
@@ -133,6 +171,13 @@ def init(*rc_files):
           "save_email": const.EMAIL,
           "alltags": "true", # show all tags, or just "human-readable" ones
           },
+
+        "albumart":
+        { "round": "true", # use rounded corners for artwork thumbnails
+          "prefer_embedded": "false",
+          "force_filename": "false",
+          "filename": "folder.jpg",
+          }
         }
 
     # <=2.2.1 QL created the user folder in the profile folder
@@ -151,6 +196,12 @@ def init(*rc_files):
 
     _config.read(rc_files)
 
+    # revision 94d389a710f1
+    from_ = ("settings", "round")
+    if _config.has_option(*from_):
+        _config.set("albumart", "round", _config.get(*from_))
+        _config.remove_option(*from_)
+
 def state(arg):
     return _config.getboolean("settings", arg)
 
