@@ -7,8 +7,10 @@ import locale
 if os.name == "nt":
     from win32com.shell import shellcon, shell
 
-VERSION_TUPLE = (2, 3, 2)
+VERSION_TUPLE = (2, 4)
 VERSION = ".".join(map(str, VERSION_TUPLE))
+PROCESS_TITLE_QL = "quodlibet"
+PROCESS_TITLE_EF = "exfalso"
 
 # expanduser doesn't work with unicode on win...
 if os.name == "nt":
@@ -106,15 +108,7 @@ NBP_EXAMPLES = """\
 /path/<artist> - <album>/<tracknumber>. <title>
 /path/<artist>/<album>/<tracknumber> - <title>"""
 
-if "QUODLIBET_DEBUG" in os.environ:
-    DEBUG = True
-    DEBUG_MENU = """\
-      <menuitem action='DebugReload'/>
-      <menuitem action='DebugCauseError'/>
-"""
-else:
-    DEBUG = False
-    DEBUG_MENU = ""
+DEBUG = "QUODLIBET_DEBUG" in os.environ
 
 MENU = """<ui>
   <menubar name='Menu'>
