@@ -11,7 +11,6 @@ import gtk
 
 from quodlibet import const
 from quodlibet import qltk
-from quodlibet import stock
 from quodlibet import util
 
 from quodlibet.devices._base import Device
@@ -25,12 +24,19 @@ class IPodSong(AudioFile):
         super(IPodSong, self).__init__()
         self.sanitize(gpod.itdb_filename_on_ipod(track))
 
-        for key in ['artist', 'album', 'title', 'genre', 'grouping']:
-            value = getattr(track, key)
+        # String keys
+        for key in ['artist', 'album', 'title', 'genre', 'grouping',
+            'composer', 'albumartist']:
+            # albumartist since libgpod-0.4.2
+            value = getattr(track, key, None)
             if value:
-                try: self[key] = unicode(value)
-                except UnicodeDecodeError:
-                    self[key] = unicode(value, errors='replace')
+                self[key] = util.decode(value)
+        # Sort keys (since libgpod-0.5.0)
+        for key in ['artist', 'album', 'albumartist']:
+            value = getattr(track, 'sort_' + key, None)
+            if value:
+                self[key + 'sort'] = util.decode(value)
+        # Numeric keys
         for key in ['bitrate', 'playcount']:
             value = getattr(track, key)
             if value:
@@ -62,7 +68,7 @@ class IPodSong(AudioFile):
         return []
 
 class IPodDevice(Device):
-    icon = stock.IPOD
+    icon = 'multimedia-player-ipod'
     protocol = 'ipod'
 
     ordered = True
@@ -78,6 +84,17 @@ class IPodDevice(Device):
     __itdb = None
     __covers = []
 
+    def __init__(self, backend_id, device_id):
+        super(IPodDevice, self).__init__(backend_id, device_id)
+
+        #In case we can't initialize the iPod and there is a rockbox directory,
+        #the device is probably a rockboxed iPod.
+        #FIXME: what if the device isn't connected?
+        self.__itdb = gpod.itdb_parse(self.mountpoint, None)
+        rockbox_dir = os.path.join(self.mountpoint, ".rockbox")
+        if self.__itdb is None and os.path.isdir(rockbox_dir):
+            raise TypeError
+
     def Properties(self):
         props = []
 
@@ -144,7 +161,7 @@ class IPodDevice(Device):
         return d
 
     def list(self, wlb):
-        self.__load_db()
+        if self.__load_db() is None: return []
         songs = []
         orphaned = False
         for track in gpod.sw_get_tracks(self.__itdb):
@@ -161,35 +178,34 @@ class IPodDevice(Device):
         return songs
 
     def copy(self, songlist, song):
-        self.__load_db()
+        if self.__load_db() is None: return False
         track = gpod.itdb_track_new()
 
+        # All values should be utf-8 encoded strings
+        # Filepaths should be encoded with the fs encoding
+
         # Either combine tags with comma, or only take the first value
         if self['all_tags']: tag = song.comma
-        else: tag = lambda key: song.list(key)[0]
+        else: tag = lambda key: (song.list(key) or ('',))[0]
 
-        try:
-            title = tag('title')
-        except IndexError:
-            pass
-        else:
-            if self['title_version'] and song('version'):
-                title = " - ".join([title, song('version')])
-            track.title = str(title)
+        title = tag('title')
+        if self['title_version'] and song('version'):
+            title = " - ".join([title, song('version')])
+        track.title = util.encode(title)
 
-        try:
-            album = tag('album')
-        except IndexError:
-            pass
-        else:
-            if self['album_part'] and song('discsubtitle'):
-                album = " - ".join([album, song('discsubtitle')])
-            track.album = str(album)
+        album = tag('album')
+        if self['album_part'] and song('discsubtitle'):
+            album = " - ".join([album, song('discsubtitle')])
+        track.album = util.encode(album)
 
         # String keys
-        for key in ['artist', 'genre', 'grouping']:
-            try: setattr(track, key, str(tag(key)))
-            except IndexError: continue
+        for key in ['artist', 'genre', 'grouping', 'composer', 'albumartist']:
+            if hasattr(track, key): # albumartist since libgpod-0.4.2
+                setattr(track, key, util.encode(tag(key)))
+        # Sort keys (since libgpod-0.5.0)
+        for key in ['artist', 'album', 'albumartist']:
+            if hasattr(track, 'sort_' + key):
+                setattr(track, 'sort_' + key, util.encode(tag(key + 'sort')))
         # Numeric keys
         for key in ['bitrate', 'playcount', 'year']:
             try: setattr(track, key, int(song('~#'+key)))
@@ -211,7 +227,7 @@ class IPodDevice(Device):
             except ValueError: continue
 
         track.filetype = song('~format')
-        track.comment = song('~filename')
+        track.comment = util.encode(util.fsdecode(song('~filename')))
 
         # Associate a cover with the track
         if self['covers']:
@@ -236,7 +252,7 @@ class IPodDevice(Device):
             return False
 
     def delete(self, songlist, song):
-        self.__load_db()
+        if self.__load_db() is None: return False
         try:
             for track in gpod.sw_get_tracks(self.__itdb):
                 if gpod.itdb_filename_on_ipod(track) == song['~filename']:
@@ -276,6 +292,7 @@ class IPodDevice(Device):
         return self.__itdb
 
     def __save_db(self):
+        if self.__itdb is None: return True
         if gpod.itdb_write(self.__itdb, None) == 1 and \
            gpod.itdb_shuffle_write(self.__itdb, None) == 1:
             return True
@@ -293,7 +310,7 @@ class IPodDevice(Device):
         return db
 
     def __close_db(self):
-        if self.__itdb: gpod.itdb_free(self.__itdb)
+        if self.__itdb is not None: gpod.itdb_free(self.__itdb)
         self.__itdb = None
 
     def __remove_track(self, track):
