@@ -38,28 +38,46 @@ class APEv2File(AudioFile):
                 key not in self.IGNORE):
                 self[key] = "\n".join(list(value))
 
+        self.sanitize(filename)
+
+    @staticmethod
+    def __titlecase(key):
+        if key.lower() in ["isrc", "isbn", "ean/upc"]: return key.upper()
+        else: return key.title()
+
     def can_change(self, key=None):
         if key is None: return True
         else: return (super(APEv2File, self).can_change(key) and
-                      key not in self.IGNORE and key not in self.TRANS)
+                      key.lower() not in self.IGNORE and
+                      key.lower() not in self.TRANS and
+                      mutagen.apev2.is_valid_apev2_key(self.__titlecase(key)))
 
     def write(self):
         try: tag = mutagen.apev2.APEv2(self['~filename'])
         except mutagen.apev2.APENoHeaderError:
             tag = mutagen.apev2.APEv2()
 
-        keys = tag.keys()
-        for key in keys:
-            # remove any text keys we read in
+        # Remove any text keys we read in
+        for key in tag.iterkeys():
             value = tag[key]
-            if (value.kind == mutagen.apev2.TEXT and key not in self.IGNORE):
+            if (value.kind == mutagen.apev2.TEXT and
+                key.lower() not in self.IGNORE):
                 del(tag[key])
+
+        # Merge all case differing keys together and titlecase them
         for key in self.realkeys():
-            if key in self.IGNORE: continue
             value = self[key]
-            key = self.SNART.get(key, key)
-            if key in ["isrc", "isbn", "ean/upc"]: key = key.upper()
-            else: key = key.title()
-            tag[key] = value.split("\n")
+            key = self.SNART.get(key.lower(), key.lower())
+            if key in self.IGNORE: continue
+            key = self.__titlecase(key)
+            if key in tag:
+                old_val = tag[key]
+                if old_val.kind == mutagen.apev2.TEXT:
+                    tag[key] = list(old_val) + value.split("\n")
+                else: # FIXME: We overwrite data here.
+                    tag[key] = value.split("\n")
+            else:
+                tag[key] = value.split("\n")
+
         tag.save(self["~filename"])
         self.sanitize()
