@@ -14,6 +14,7 @@ import cPickle as pickle
 import itertools
 import os
 import shutil
+import threading
 
 import gobject
 import gtk
@@ -24,7 +25,8 @@ try:
 except ImportError:
     fcntl = None
 
-from quodlibet import util
+from quodlibet import util, print_d, print_w
+from quodlibet.qltk.msg import ErrorMessage
 
 class Library(gtk.Object):
     """A Library contains useful objects.
@@ -48,6 +50,7 @@ class Library(gtk.Object):
 
     def __init__(self, name=None):
         super(Library, self).__init__()
+        self._save_lock = threading.Lock()
         self._contents = {}
         self._masked = {}
         for key in ['get', 'keys', 'values', 'items', 'iterkeys',
@@ -65,10 +68,11 @@ class Library(gtk.Object):
         items = filter(lambda item: item not in self, items)
         if not items:
             return
+
         print_d("Adding %d items." % len(items), self)
         for item in items:
-            print_d("Adding %r." % item.key, self)
             self._contents[item.key] = item
+
         self.dirty = True
         self.emit('added', items)
         return items
@@ -77,10 +81,11 @@ class Library(gtk.Object):
         """Remove items. This causes a 'removed' signal."""
         if not items:
             return
+
         print_d("Removing %d items." % len(items), self)
         for item in items:
-            print_d("Removing %r." % item.key, self)
             del(self._contents[item.key])
+
         self.dirty = True
         self.emit('removed', items)
 
@@ -143,8 +148,11 @@ class Library(gtk.Object):
         print_d("Loading contents of %r." % filename, self)
         try:
             if os.path.exists(filename):
+                # pickle makes 1000 read syscalls for 6000 songs
+                # read the file into memory so that there are less
+                # context switches. saves 40% here..
                 fileobj = file(filename, "rb")
-                try: items = pickle.load(fileobj)
+                try: items = pickle.loads(fileobj.read())
                 except (pickle.PickleError, EnvironmentError,
                         ImportError, EOFError):
                     util.print_exc()
@@ -174,11 +182,21 @@ class Library(gtk.Object):
 
     def save(self, filename=None):
         """Save the library to the given filename."""
+        self._save_lock.acquire()
         if filename is None:
             filename = self.filename
         print_d("Saving contents to %r." % filename, self)
         if not os.path.isdir(os.path.dirname(filename)):
             os.makedirs(os.path.dirname(filename))
+        # Issue 479. Catch problem early
+        if os.path.isdir(filename):
+            msg = _("Cannot save library contents to %s (it's a directory). "
+                    "Please remove it and try again.") % filename
+            print_w(msg)
+            # TODO: Better handling of this edge-case...
+            ErrorMessage(None, _("Library Error"), msg).run()
+            self._save_lock.release()
+            return
         fileobj = file(filename + ".tmp", "wb")
         if fcntl is not None:
             fcntl.flock(fileobj.fileno(), fcntl.LOCK_EX)
@@ -189,7 +207,13 @@ class Library(gtk.Object):
         # sorting takes advantage of the filesystem cache when we
         # reload/rescan the files.
         items.sort(key=lambda item: item.key)
-        pickle.dump(items, fileobj, pickle.HIGHEST_PROTOCOL)
+        # While protocol 2 is usually faster it uses __setitem__
+        # for unpickle and we override it to clear the sort cache.
+        # This roundtrip makes it much slower, so we use protocol 1
+        # unpickle numbers (py2.7):
+        #   2: 0.66s / 2 + __set_item__: 1.18s / 1 + __set_item__: 0.72s
+        # see: http://bugs.python.org/issue826897
+        pickle.dump(items, fileobj, 1)
         fileobj.flush()
         os.fsync(fileobj.fileno())
         fileobj.close()
@@ -199,6 +223,7 @@ class Library(gtk.Object):
         os.rename(filename + ".tmp", filename)
         self.dirty = False
         print_d("Done saving contents to %r." % filename, self)
+        self._save_lock.release()
 
 class Librarian(gtk.Object):
     """The librarian is a nice interface to all active libraries.
