@@ -1,4 +1,5 @@
 # Copyright 2006 Joe Wreschnig
+#           2011 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -374,7 +375,7 @@ class FileLibrary(Library):
         for value in self.scan(paths, exclude, cofuncid):
             yield value
 
-    def add_filename(self, filename, signal=True):
+    def add_filename(self, filename, add=True):
         """Add a file based on its filename.
 
         Subclasses must override this to open the file correctly.
@@ -406,12 +407,12 @@ class FileLibrary(Library):
                                 if item is not None:
                                     added.append(item)
                                     if len(added) > 20:
-                                        self.emit('added', added)
+                                        self.add(added)
                                         added = []
                                         task.pulse()
                                         yield True
                     if added:
-                        self.emit('added', added)
+                        self.add(added)
                         added = []
                         task.pulse()
                         yield True
@@ -447,25 +448,26 @@ class FileLibrary(Library):
 class SongFileLibrary(SongLibrary, FileLibrary):
     """A library containing song files."""
 
-    def add_filename(self, filename, signal=True):
+    def add_filename(self, filename, add=True):
         """Add a song to the library based on filename.
 
-        If 'signal' is true, the 'added' signal may fire.
+        If 'add' is true, the song will be added and the 'added' signal
+        may be fired.
 
-        If the song was added, it is returned. Otherwise, None
-        is returned.
+        Example (add=False):
+            load many songs and call Library.add(songs) to add all in one go.
+
+        The song is returned if it is in the library after this call.
+        Otherwise, None is returned.
         """
-        try:
-            if filename not in self._contents:
-                song = MusicFile(filename)
-                if song:
-                    print_d("Adding %r based on filename." % filename, self)
-                    self.dirty = True
-                    self._contents[song.key] = song
-                    if signal:
-                        self.add([song])
-                    return song
-            else:
-                return self._contents[filename]
-        except StandardError:
-            util.print_exc()
+
+        song = None
+        if filename not in self._contents:
+            song = MusicFile(filename)
+            if song and add:
+                self.add([song])
+        else:
+            print_d("Already got file %r." % filename)
+            song = self._contents[filename]
+
+        return song
