@@ -12,13 +12,12 @@ import re
 
 from quodlibet.parse import _match as match
 from quodlibet.parse._scanner import Scanner
+from quodlibet.parse._match import error, ParseError
 
 # Token types.
 (NEGATION, INTERSECT, UNION, OPENP, CLOSEP, EQUALS, OPENRE,
  CLOSERE, REMODS, COMMA, TAG, RE, RELOP, NUMCMP, EOF) = range(15)
 
-class error(ValueError): pass
-class ParseError(error): pass
 class LexerError(error): pass
 
 class QueryLexer(Scanner):
@@ -86,7 +85,10 @@ class QueryParser(object):
     def _match_parened(self, expect, ReturnType, InternalType):
         self.match(expect)
         self.match(OPENP)
-        m = ReturnType(InternalType())
+        m = InternalType()
+        if len(m) > 1:
+            m = ReturnType(m)
+        else: m = m[0]
         self.match(CLOSEP)
         return m
 
@@ -158,6 +160,8 @@ class QueryParser(object):
 
     def QueryPart(self):
         names = [s.lower() for s in self._match_list(self._match_string)]
+        if filter(lambda k: k.encode("ascii", "replace") != k, names):
+            raise ParseError("Expected ascii key")
         self.match(EQUALS)
         res = self.RegexpSet()
         return match.Tag(names, res)
@@ -252,20 +256,22 @@ def _expand_query(string, tags):
 
 STAR = ["artist", "album", "title"]
 def Query(string, star=STAR):
-    string = string.strip()
     if not isinstance(string, unicode): string = string.decode('utf-8')
-    if string == "": return match.Inter([])
+    if Query.match_all(string): return match.Inter([])
     string = _expand_query(string, star)
     return QueryParser(QueryLexer(string)).StartQuery()
 Query.STAR = STAR
 
 def is_valid(string):
-    if string.strip() == "": return True
+    if Query.match_all(string): return True
     try: QueryParser(QueryLexer(string)).StartQuery()
     except error: return False
     else: return True
 Query.is_valid = is_valid
 
+def match_all(string):
+    return string.strip() == ""
+Query.match_all = match_all
 
 def is_parsable(string):
     string = _expand_query(string, ["foo"])
