@@ -17,15 +17,19 @@ class BasePlayer(gtk.Object):
     change things, use .song.
     """
 
-    _paused = False
-    paused = False
+    name = ""
+    version_info = ""
+
+    paused = None
     song = None
     info = None
+    volume = None
+
     # Replay Gain profiles are a list of values to be tried in order;
-    # Three things can set them: play order, browser, and a default.
-    replaygain_profiles = [None, None, ["none"]]
-    _length = 1
+    # Four things can set them: rg menu, browser, play order, and a default.
+    replaygain_profiles = [None, None, None, ["none"]]
     _volume = 1.0
+    _paused = True
     _eq_values = []
 
     _gsignals_ = {
@@ -46,18 +50,9 @@ class BasePlayer(gtk.Object):
                    0.0, 1.0, 1.0, gobject.PARAM_READWRITE)
         }
 
-
-
     def __init__(self, *args, **kwargs):
         super(BasePlayer, self).__init__()
 
-    def do_song_started(self, song):
-        # Reset Replay Gain levels based on the new song.
-        self.volume = self.volume
-
-    def do_song_ended(self, song, stopped):
-        self.volume = self.volume
-
     def do_get_property(self, property):
         if property.name == 'volume':
             return self._volume
@@ -96,10 +91,10 @@ class BasePlayer(gtk.Object):
         if self.song:
             self.paused = False
 
-    def previous(self):
+    def previous(self, force=False):
         # Go back if standing at the beginning of the song,
         # otherwise restart the current song.
-        if self.get_position() < 1000:
+        if force or self.get_position() < 1500:
             self._source.previous()
             self._end(True)
         else:
@@ -108,7 +103,7 @@ class BasePlayer(gtk.Object):
             self.paused = False
 
     def go_to(self, song, explicit=False):
-        print_d("Going to %r" % song, context=self)
+        print_d("Going to %r" % getattr(song, "key", song))
         res = self._source.go_to(song, explicit)
         if explicit and not res:
             return False
