@@ -192,6 +192,7 @@ class XinePlaylistPlayer(BasePlayer):
         self.emit('song-started', self.song)
 
         if self.song is not None:
+            self.volume = self.volume
             if gapless and self._supports_gapless:
                 xine_set_param(self._stream, XINE_PARAM_GAPLESS_SWITCH, 1)
             xine_open(self._stream, self.song("~uri"))
@@ -219,9 +220,14 @@ class XinePlaylistPlayer(BasePlayer):
     def update_eq_values(self):
         from quodlibet.player import _xine as _xine_module
         bands = self.eq_bands
+        need_eq = any(self._eq_values)
         for band, val in enumerate(self._eq_values):
             param = getattr(_xine_module, 'XINE_PARAM_EQ_%dHZ' % bands[band])
-            xine_set_param(self._stream, param, int(val*100/24.))
+            # between 1..200; 100 is the default gain; 0 means no EQ filter
+            # only negative gain seems to work
+            val = (int(val * 100 / 24.0) + 100) or 1
+            val *= int(need_eq)
+            xine_set_param(self._stream, param, val)
 
 _xine = None
 _plugins = None
