@@ -12,22 +12,24 @@ from quodlibet import const
 from quodlibet import formats
 from quodlibet.util import fver
 
-class AboutQuodLibet(gtk.AboutDialog):
-    def __init__(self, parent, player):
-        super(AboutQuodLibet, self).__init__()
+class AboutDialog(gtk.AboutDialog):
+    def __init__(self, parent, player, name):
+        super(AboutDialog, self).__init__()
         self.set_transient_for(parent)
-        self.set_name("Quod Libet")
+        self.set_program_name(name)
         self.set_version(const.VERSION)
         self.set_authors(const.AUTHORS)
         self.set_artists(const.ARTISTS)
         fmts = ", ".join(formats.modules)
         text = []
         text.append(_("Supported formats: %s") % fmts)
-        text.append(_("Audio device: %s") % player.name)
+        if player:
+            text.append(_("Audio device: %s") % player.name)
         text.append("Mutagen: %s" % fver(mutagen.version))
         text.append("GTK+: %s / PyGTK: %s" %(
             fver(gtk.gtk_version), fver(gtk.pygtk_version)))
-        text.append(player.version_info)
+        if player:
+            text.append(player.version_info)
         self.set_comments("\n".join(text))
         # Translators: Replace this with your name/email to have it appear
         # in the "About" dialog.
@@ -38,7 +40,10 @@ class AboutQuodLibet(gtk.AboutDialog):
             "<quod-libet-development@googlegroups.com>")
         self.child.show_all()
 
-def show(window, player):
-    about = AboutQuodLibet(window, player)
-    about.run()
-    about.destroy()
+class AboutQuodLibet(AboutDialog):
+    def __init__(self, parent, player):
+        super(AboutQuodLibet, self).__init__(parent, player, "Quod Libet")
+
+class AboutExFalso(AboutDialog):
+    def __init__(self, parent, player=None):
+        super(AboutExFalso, self).__init__(parent, player, "Ex Falso")
