@@ -1,4 +1,4 @@
-# Copyright 2005 Joe Wreschnig, Michael Urman
+# Copyright 2005-2011 Joe Wreschnig, Michael Urman, Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -17,18 +17,12 @@ if not getattr(gtk.Entry, "set_icon_from_stock", None):
     except ImportError: pass
 
 from quodlibet import config
-from quodlibet.qltk import ClearButton, get_top_parent
+from quodlibet.qltk.x import ClearButton, is_accel
 
 class EditableUndo(object):
     """A simple undo/redo implementation for gtk widgets that
     support the gtk.Editable interface"""
 
-    __gsignals__ = {
-        "undo": (
-        gobject.SIGNAL_RUN_LAST|gobject.SIGNAL_ACTION, gobject.TYPE_NONE, ()),
-        "redo": (
-        gobject.SIGNAL_RUN_LAST|gobject.SIGNAL_ACTION, gobject.TYPE_NONE, ())}
-
     def set_undo(self, val):
         if val: self.__enable_undo()
         else: self.__disable_undo()
@@ -57,21 +51,19 @@ class EditableUndo(object):
 
         self.__handlers = [
             self.connect("insert-text", self.__insert_before),
-            self.connect("delete-text", self.__delete_before)]
+            self.connect("delete-text", self.__delete_before),
+            self.connect('populate-popup', self.__popup),
+            self.connect('key-press-event', self.__key_press),
+            ]
 
-        self.__accels = gtk.AccelGroup()
-        self.add_accelerator('undo', self.__accels, ord('z'),
-            gtk.gdk.CONTROL_MASK, 0)
-        self.add_accelerator('redo', self.__accels, ord('z'),
-            gtk.gdk.SHIFT_MASK | gtk.gdk.CONTROL_MASK, 0)
-
-        self.__rlz = self.connect('realize', self.__realize)
-
-        self.__handlers.extend([
-            self.connect('undo', lambda *x: self.undo()),
-            self.connect('redo', lambda *x: self.redo()),
-            self.connect('populate-popup', self.__popup)
-            ])
+    def __key_press(self, entry, event):
+        if is_accel(event, "<ctrl>z"):
+            self.undo()
+            return True
+        elif is_accel(event, "<ctrl><shift>z"):
+            self.redo()
+            return True
+        return False
 
     def __disable_undo(self):
         for handler in self.__handlers:
@@ -83,12 +75,6 @@ class EditableUndo(object):
         del self.__in_pos
         del self.__del_pos
 
-    def __realize(self, *args):
-        self.disconnect(self.__rlz)
-        del self.__rlz
-        parent = get_top_parent(self)
-        parent.add_accel_group(self.__accels)
-
     def __popup(self, entry, menu):
         undo =  gtk.ImageMenuItem(gtk.STOCK_UNDO)
         redo =  gtk.ImageMenuItem(gtk.STOCK_REDO)
@@ -159,7 +145,6 @@ class EditableUndo(object):
         self.__uninhibit()
 
 class UndoEntry(IconEntry, EditableUndo):
-    __gsignals__ = EditableUndo.__gsignals__
     def __init__(self, *args, **kwargs):
         super(UndoEntry, self).__init__(*args, **kwargs)
         self.set_undo(True)
@@ -169,7 +154,6 @@ class UndoEntry(IconEntry, EditableUndo):
         self.reset_undo()
 
 class UndoNoSexyEntry(gtk.Entry, EditableUndo):
-    __gsignals__ = EditableUndo.__gsignals__
     def __init__(self, *args):
         super(UndoNoSexyEntry, self).__init__(*args)
         self.set_undo(True)
@@ -179,19 +163,37 @@ class UndoNoSexyEntry(gtk.Entry, EditableUndo):
         self.reset_undo()
 
 class ClearEntryMixin(object):
+    """A clear icon mixin supporting newer gtk.Entry or sexy.IconEntry /
+    a separate clear button as a fallback.
+    """
+
+    __gsignals__ = {'clear': (
+        gobject.SIGNAL_RUN_LAST|gobject.SIGNAL_ACTION, gobject.TYPE_NONE, ())}
+
     def pack_clear_button(self, container=None):
+        """Enables the clear icon in the entry. For older gtk+
+        versions and without libsexy, a clear button will be packed in
+        container.
+        """
         if getattr(self, "set_icon_from_stock", None):
             self.set_icon_from_stock(
                 gtk.ENTRY_ICON_SECONDARY, gtk.STOCK_CLEAR)
-            clear = lambda *x: x[0].delete_text(0, -1)
-            self.connect("icon-release", clear)
-        elif getattr(self, "add_clear_button", None):
-            self.add_clear_button()
-        else:
-            container.pack_start(ClearButton(self), False)
+            self.connect("icon-release", self.__clear)
+        elif container:
+            button = ClearButton(self)
+            container.pack_start(button, False)
+            button.connect('clicked', self.__clear)
 
-class ClearEntry(UndoEntry, ClearEntryMixin): pass
-class ClearNoSexyEntry(UndoNoSexyEntry, ClearEntryMixin): pass
+    def __clear(self, button, *args):
+        # TODO: don't change the order.. we connect to clear and remove all
+        # timeouts added for text change in the searchbar
+        self.delete_text(0, -1)
+        self.emit('clear')
+
+class ClearEntry(UndoEntry, ClearEntryMixin):
+    __gsignals__ = ClearEntryMixin.__gsignals__
+class ClearNoSexyEntry(UndoNoSexyEntry, ClearEntryMixin):
+    __gsignals__ = ClearEntryMixin.__gsignals__
 
 class ValidatingEntryMixin(object):
     """An entry with visual feedback as to whether it is valid or not.
