@@ -26,6 +26,8 @@ from quodlibet.qltk.pluginwin import PluginWindow
 from quodlibet.qltk.renamefiles import RenameFiles
 from quodlibet.qltk.tagsfrompath import TagsFromPath
 from quodlibet.qltk.tracknumbers import TrackNumbers
+from quodlibet.qltk.entry import UndoEntry
+from quodlibet.qltk.about import AboutExFalso
 from quodlibet.plugins.editing import EditingPlugins
 from quodlibet.plugins.songsmenu import SongsMenuPlugins
 
@@ -60,11 +62,19 @@ class ExFalsoWindow(gtk.Window):
         vb = gtk.VBox(spacing=6)
 
         bbox = gtk.HBox(spacing=6)
+
+        about = gtk.Button()
+        about.add(gtk.image_new_from_stock(
+            gtk.STOCK_ABOUT, gtk.ICON_SIZE_BUTTON))
+        about.connect_object('clicked', self.__show_about, self)
+        bbox.pack_start(about, expand=False)
+
         prefs = gtk.Button()
         prefs.add(gtk.image_new_from_stock(
             gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_BUTTON))
         prefs.connect_object('clicked', PreferencesWindow, self)
         bbox.pack_start(prefs, expand=False)
+
         plugins = gtk.Button(stock=stock.PLUGINS)
         plugins.connect_object('clicked', PluginWindow, self)
         bbox.pack_start(plugins, expand=False)
@@ -73,24 +83,8 @@ class ExFalsoWindow(gtk.Window):
         l.set_alignment(1.0, 0.5)
         l.set_ellipsize(pango.ELLIPSIZE_END)
         bbox.pack_start(l)
-        
-        if os.name == "nt":
-            # use HOME, Documents (if not under HOME), and non-floppy drives
-            folders = [const.HOME]
-            try: from win32com.shell import shell, shellcon as con
-            except ImportError, e: pass
-            else:
-                documents = shell.SHGetFolderPath(0, con.CSIDL_PERSONAL, 0, 0)
-                if not documents.startswith(const.HOME + os.sep):
-                    folders.append(documents)
-                music = shell.SHGetFolderPath(0, con.CSIDL_MYMUSIC, 0, 0)
-                folders.insert(0, music)
-            folders = filter(os.path.isdir, folders +
-                [letter + ":\\" for letter in "CDEFGHIJKLMNOPQRSTUVWXYZ"])
-        else:
-            folders = [const.HOME, "/"]
 
-        fs = FileSelector(dir, folders=folders)
+        fs = FileSelector(dir)
 
         vb.pack_start(fs)
         vb.pack_start(bbox, expand=False)
@@ -119,7 +113,7 @@ class ExFalsoWindow(gtk.Window):
 
         self.__ag = gtk.AccelGroup()
         key, mod = gtk.accelerator_parse("<control>Q")
-        self.__ag.connect_group(key, mod, 0, gtk.main_quit)
+        self.__ag.connect_group(key, mod, 0, lambda *x: self.destroy())
         self.add_accel_group(self.__ag)
 
         self.connect('configure-event', ExFalsoWindow.__save_size)
@@ -130,10 +124,14 @@ class ExFalsoWindow(gtk.Window):
             self.maximize()
         self.resize(*map(int, config.get("memory", "exfalso_size").split()))
 
+    def __show_about(self, window):
+        about = AboutExFalso(self)
+        about.run()
+        about.destroy()
+
     def __window_state_changed(self, window, event):
         self.__state = event.new_window_state
         if self.__state & gtk.gdk.WINDOW_STATE_WITHDRAWN: return
-        print int(self.__state & gtk.gdk.WINDOW_STATE_MAXIMIZED)
         if self.__state & gtk.gdk.WINDOW_STATE_MAXIMIZED:
             config.set("memory", "exfalso_maximized", "1")
         else:
@@ -226,7 +224,7 @@ class PreferencesWindow(qltk.UniqueWindow):
 
         vbox = gtk.VBox(spacing=6)
         hb = gtk.HBox(spacing=6)
-        e = gtk.Entry()
+        e = UndoEntry()
         e.set_text(config.get("editing", "split_on"))
         e.connect('changed', self.__changed, 'editing', 'split_on')
         l = gtk.Label(_("Split _on:"))
@@ -240,7 +238,17 @@ class PreferencesWindow(qltk.UniqueWindow):
         vbox.pack_start(hb, expand=False)
         vbox.pack_start(cb, expand=False)
         f = qltk.Frame(_("Tag Editing"), child=vbox)
-        self.add(f)
+
+        close = gtk.Button(stock=gtk.STOCK_CLOSE)
+        close.connect_object('clicked', lambda x: x.destroy(), self)
+        button_box = gtk.HButtonBox()
+        button_box.set_layout(gtk.BUTTONBOX_END)
+        button_box.pack_start(close)
+
+        main_vbox = gtk.VBox(spacing=12)
+        main_vbox.pack_start(f)
+        main_vbox.pack_start(button_box, expand=False)
+        self.add(main_vbox)
 
         self.connect_object('destroy', PreferencesWindow.__destroy, self)
         self.show_all()
