@@ -7,6 +7,7 @@
 
 import os
 import urlparse
+import errno
 
 import gobject
 import gtk
@@ -45,7 +46,7 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
                 os.path.basename(value) or value))
     cell_data = staticmethod(cell_data)
 
-    def __init__(self, initial=None, folders=[const.HOME, "/"]):
+    def __init__(self, initial=None, folders=None):
         super(DirectoryTree, self).__init__(gtk.TreeStore(str))
         column = gtk.TreeViewColumn(_("Folders"))
         column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
@@ -60,6 +61,26 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
         self.append_column(column)
         self.set_search_equal_func(search_func, True)
 
+        if folders is None:
+            folders = []
+
+        if os.name == "nt":
+            try: from win32com.shell import shell, shellcon as con
+            except ImportError, e: pass
+            else:
+                if folders: folders.append(None)
+                desktop = shell.SHGetFolderPath(0, con.CSIDL_DESKTOP, 0, 0)
+                folders.append(desktop)
+                folders.append(const.HOME)
+                music = shell.SHGetFolderPath(0, con.CSIDL_MYMUSIC, 0, 0)
+                folders.append(music)
+            if folders: folders.append(None)
+            drives = [letter + ":\\" for letter in "CDEFGHIJKLMNOPQRSTUVWXYZ"]
+            map(folders.append, filter(os.path.isdir, drives))
+        else:
+            if folders: folders.append(None)
+            folders.extend([const.HOME, "/"])
+
         # Read in the GTK bookmarks list; gjc says this is the right way
         try: f = file(os.path.join(const.HOME, ".gtk-bookmarks"))
         except EnvironmentError: pass
@@ -134,7 +155,7 @@ class DirectoryTree(RCMTreeView, MultiDragTreeView):
         delete = menu.get_children()[1]
         try: delete.set_sensitive(len(os.listdir(util.fsnative(directory))) == 0)
         except OSError, err:
-            if err.errno == 2: model.remove(model.get_iter(path))
+            if err.errno == errno.ENOENT: model.remove(model.get_iter(path))
         else:
             selection = self.get_selection()
             selection.unselect_all()
@@ -258,8 +279,7 @@ class FileSelector(gtk.VPaned):
                                  gobject.TYPE_NONE, (gtk.TreeSelection,))
                      }
 
-    def __init__(self, initial=None, filter=filesel_filter,
-                 folders=[const.HOME, "/"]):
+    def __init__(self, initial=None, filter=filesel_filter, folders=None):
         super(FileSelector, self).__init__()
         self.__filter = filter
 
@@ -322,8 +342,8 @@ class FileSelector(gtk.VPaned):
         dirs = [dmodel[row][0] for row in rows]
         for dir in dirs:
             try:
-                for file in filter(self.__filter,
-                                   sorted(os.listdir(util.fsencode(dir)))):
+                files = filter(self.__filter, util.listdir(dir))
+                for file in sorted(files):
                     filename = os.path.join(dir, file)
                     if (os.access(filename, os.R_OK) and
                             not os.path.isdir(filename)):
