@@ -119,13 +119,13 @@ class OneSong(qltk.Notebook):
 
     def _title(self, song, box):
         l = Label()
-        text = "<big><b>%s</b></big>" % util.escape(song("title"))
+        text = "<big><b>%s</b></big>" % util.escape(song.comma("title"))
         if "version" in song:
             text += "\n" + util.escape(song.comma("version"))
         l.set_markup(text)
         l.set_ellipsize(pango.ELLIPSIZE_END)
         box.pack_start(l, expand=False, fill=False)
-        self.title = song("title")
+        self.title = song.comma("title")
 
     def _album(self, song, box):
         if "album" not in song: return
@@ -557,7 +557,7 @@ class ManySongs(qltk.Notebook):
 
 class Information(Window):
     def __init__(self, library, songs, parent=None):
-        super(Information, self).__init__()
+        super(Information, self).__init__(dialog=False)
         self.set_border_width(12)
         self.set_default_size(400, 400)
         if len(songs) > 1:
