@@ -23,10 +23,19 @@ from quodlibet.qltk.songlist import SongList
 
 class PreferencesWindow(qltk.UniqueWindow):
     class SongList(gtk.VBox):
+        name = "songlist"
         def __init__(self):
             super(PreferencesWindow.SongList, self).__init__(spacing=12)
             self.set_border_width(12)
             self.title = _("Song List")
+
+            c = ConfigCheckButton(
+                _("_Jump to playing song automatically"), 'settings', 'jump')
+            c.set_tooltip_text(_("When the playing song changes, "
+                                 "scroll to it in the song list"))
+            c.set_active(config.getboolean("settings", "jump"))
+            self.pack_start(c, expand=False)
+
             vbox = gtk.VBox(spacing=12)
 
             buttons = {}
@@ -123,6 +132,7 @@ class PreferencesWindow(qltk.UniqueWindow):
             SongList.set_all_column_headers(headers)
 
     class Browsers(gtk.VBox):
+        name = "browser"
         def __init__(self):
             super(PreferencesWindow.Browsers, self).__init__(spacing=12)
             self.set_border_width(12)
@@ -181,46 +191,15 @@ class PreferencesWindow(qltk.UniqueWindow):
             config.set(section, name, entry.get_text())
 
     class Player(gtk.VBox):
+        name = "playback"
         def __init__(self):
             super(PreferencesWindow.Player, self).__init__(spacing=12)
             self.set_border_width(12)
-            self.title = _("Player")
+            self.title = _("Playback")
 
-            if config.get('player', 'backend') == "gstbe":
-                e = UndoEntry()
-                e.set_tooltip_text(_("The GStreamer output pipeline used for "
-                        "playback, such as 'alsasink device=default'. "
-                        "Leave blank for default pipeline."))
-                e.set_text(config.get('player', 'gst_pipeline'))
-                def changed(entry):
-                    config.set('player', 'gst_pipeline', entry.get_text())
-                e.connect('changed', changed)
-                l = gtk.Label(_('_Output pipeline:'))
-                l.set_use_underline(True)
-                l.set_mnemonic_widget(e)
-                b = gtk.Button(stock=gtk.STOCK_APPLY)
-                b.connect('clicked',
-                        lambda w: player.device.go_to(player.device.song))
-
-                hb = gtk.HBox(spacing=6)
-                hb.pack_start(l, expand=False)
-                hb.pack_start(e)
-                hb.pack_start(b, expand=False)
-                self.pack_start(hb, expand=False)
-
-            c = ConfigCheckButton(
-                _("_Jump to playing song automatically"), 'settings', 'jump')
-            c.set_tooltip_text(_("When the playing song changes, "
-                                 "scroll to it in the song list"))
-            c.set_active(config.getboolean("settings", "jump"))
-            self.pack_start(c, expand=False)
-
-            c = ConfigCheckButton(
-                _("_Use rounded corners on thumbnails"), 'settings', 'round')
-            c.set_tooltip_text(_("Round the corners of album artwork "
-                "thumbnail images. May require restart to take effect."))
-            c.set_active(config.getboolean("settings", "round"))
-            self.pack_start(c, expand=False)
+            if player.backend and hasattr(player.device, 'PlayerPreferences'):
+                player_prefs = player.device.PlayerPreferences()
+                self.pack_start(player_prefs, expand=False)
 
             vbox = gtk.VBox(spacing=6)
             c = ConfigCheckButton(_("_Enable Replay Gain volume adjustment"),
@@ -276,6 +255,7 @@ class PreferencesWindow(qltk.UniqueWindow):
             player.playlist.volume = player.playlist.volume
 
     class Library(gtk.VBox):
+        name = "library"
         def __init__(self):
             super(PreferencesWindow.Library, self).__init__(spacing=12)
             self.set_border_width(12)
@@ -362,6 +342,60 @@ class PreferencesWindow(qltk.UniqueWindow):
         def __changed(self, entry, section, name):
             config.set(section, name, entry.get_text())
 
+
+    class AlbumArt(gtk.VBox):
+        name = "albumart"
+        def __init__(self):
+            super(PreferencesWindow.AlbumArt, self).__init__(spacing=12)
+            self.set_border_width(12)
+            self.title = _("Album Art")
+
+            vb = gtk.VBox(spacing=6)
+            display_frame = qltk.Frame(_("Display"), child=vb)
+            c = ConfigCheckButton(
+                _("_Use rounded corners on thumbnails"), 'albumart', 'round')
+            c.set_tooltip_text(_("Round the corners of album artwork "
+                    "thumbnail images. May require restart to take effect."))
+            c.set_active(config.getboolean('albumart', 'round'))
+            vb.pack_start(c, expand=False)
+
+            # Filename choice algorithm config
+            vb = gtk.VBox(spacing=6)
+            file_frame = qltk.Frame(_("Art sources"), child=vb)
+            cb = ConfigCheckButton(
+                    _("Prefer _embedded art"), 'albumart', 'prefer_embedded')
+            cb.set_tooltip_text(_("Choose to use artwork embedded in the audio "
+                    "(where available) over other sources"))
+            cb.set_active(config.getboolean('albumart', 'prefer_embedded'))
+            vb.pack_start(cb, expand=False)
+
+            hb = gtk.HBox(spacing=3)
+            cb = ConfigCheckButton(
+                    _("_Force image filename:"), 'albumart', 'force_filename')
+            cb.set_active(config.getboolean('albumart', 'force_filename'))
+            hb.pack_start(cb, expand=False)
+
+            entry = UndoEntry()
+            entry.set_tooltip_text(
+                    _("The album art image file to use when forced"))
+            entry.set_text(config.get("albumart", "filename"))
+            entry.connect('changed', self.__changed_text, 'filename')
+            # Disable entry when not forcing
+            entry.set_sensitive(cb.get_active())
+            cb.connect('toggled', self.__toggled_force_filename, entry)
+            hb.pack_start(entry)
+            vb.pack_start(hb, expand=False)
+
+            self.pack_start(display_frame, expand=False)
+            self.pack_start(file_frame, expand=False)
+
+        def __changed_text(self, entry, name):
+            config.set('albumart', name, entry.get_text())
+
+        def __toggled_force_filename(self, cb, fn_entry):
+            fn_entry.set_sensitive(cb.get_active())
+
+
     def __init__(self, parent):
         if self.is_not_unique(): return
         super(PreferencesWindow, self).__init__()
@@ -370,13 +404,30 @@ class PreferencesWindow(qltk.UniqueWindow):
         self.set_resizable(False)
         self.set_transient_for(qltk.get_top_parent(parent))
 
-        self.add(qltk.Notebook())
-        for Page in [self.SongList, self.Browsers, self.Player, self.Library]:
-            self.child.append_page(Page())
+        self.__notebook = notebook = qltk.Notebook()
+        for Page in [self.SongList, self.Browsers, self.AlbumArt, self.Player,
+            self.Library]: notebook.append_page(Page())
+
+        close = gtk.Button(stock=gtk.STOCK_CLOSE)
+        close.connect_object('clicked', lambda x: x.destroy(), self)
+        button_box = gtk.HButtonBox()
+        button_box.set_layout(gtk.BUTTONBOX_END)
+        button_box.pack_start(close)
+
+        vbox = gtk.VBox(spacing=12)
+        vbox.pack_start(notebook)
+        vbox.pack_start(button_box, expand=False)
+        self.add(vbox)
 
         self.connect_object('destroy', PreferencesWindow.__destroy, self)
         self.show_all()
 
+    def set_page(self, name):
+        notebook = self.__notebook
+        for p in range(notebook.get_n_pages()):
+            if notebook.get_nth_page(p).name == name:
+                notebook.set_current_page(p)
+
     def __destroy(self):
         config.write(const.CONFIG)
 
