@@ -12,6 +12,7 @@ import gtk
 from quodlibet import config
 from quodlibet import const
 from quodlibet import util
+from quodlibet import qltk
 
 from quodlibet.qltk.ccb import ConfigCheckButton
 from quodlibet.qltk.songlist import SongList
@@ -162,8 +163,8 @@ class PlayQueue(SongList):
         self.connect('key-press-event', self.__delete_key_pressed)
 
     def __delete_key_pressed(self, widget, event):
-        if (event.keyval, event.state) == gtk.accelerator_parse("Delete"):
-            self.__remove(None)
+        if qltk.is_accel(event, "Delete"):
+            self.__remove()
             return True
         return False
 
@@ -202,9 +203,10 @@ class PlayQueue(SongList):
         menu.show_all()
         return self.popup_menu(menu, 0, gtk.get_current_event_time())
 
-    def __remove(self, item):
+    def __remove(self, *args):
         model, paths = self.get_selection().get_selected_rows()
-        if model: map(self.model.remove, map(model.get_iter, paths))
+        if not paths: return
+        self._remove(map(model.get_iter, paths))
 
     def set_sort_by(self, *args): pass
     def get_sort_by(self, *args): return "", False
