@@ -19,7 +19,6 @@ from quodlibet import const
 from quodlibet import formats
 from quodlibet import player
 from quodlibet import qltk
-from quodlibet.qltk import about
 from quodlibet import stock
 from quodlibet import util
 
@@ -43,7 +42,8 @@ from quodlibet.qltk.prefs import PreferencesWindow
 from quodlibet.qltk.queue import QueueExpander
 from quodlibet.qltk.songlist import SongList, PlaylistMux
 from quodlibet.qltk.x import RPaned
-from quodlibet.util import copool
+from quodlibet.qltk.about import AboutQuodLibet
+from quodlibet.util import copool, gobject_weak
 from quodlibet.util.uri import URI
 from quodlibet.util.library import background_filter
 
@@ -83,7 +83,7 @@ class MainSongList(SongList):
     _activated = False
 
     def __init__(self, library, player):
-        super(MainSongList, self).__init__(library, player)
+        super(MainSongList, self).__init__(library, player, update=True)
 
         self.connect_object('row-activated', self.__select_song, player)
 
@@ -184,9 +184,8 @@ class QuodLibetWindow(gtk.Window):
         align = gtk.Alignment(xscale=1, yscale=1)
         align.set_padding(0, 6, 6, 6)
         hbox = gtk.HBox(spacing=12)
-        hb = gtk.HBox(spacing=3)
+        hb = gtk.HBox(spacing=6)
         label = gtk.Label(_("_Order:"))
-        label.set_size_request(-1, 24)
         self.order = order = PlayOrder(self.songlist.model, player)
         label.set_mnemonic_widget(order)
         label.set_use_underline(True)
@@ -225,8 +224,16 @@ class QuodLibetWindow(gtk.Window):
         hbox.pack_start(self.statusbar)
         self.statusbar.show()
 
-        self.select_browser(
-            self, config.get("memory", "browser"), library, player, True)
+        try:
+            self.select_browser(
+                self, config.get("memory", "browser"), library, player, True)
+        except:
+            config.set("memory", "browser", browsers.name(0))
+            config.save(const.CONFIG)
+            raise
+
+        # set at least the playlist before the mainloop starts..
+        player.setup(self.playlist, None, 0)
 
         def delayed_song_set():
             song = library.get(config.get("memory", "song"))
@@ -252,13 +259,20 @@ class QuodLibetWindow(gtk.Window):
         self.songlist.get_selection().connect(
                 'changed', util.DeferredSignal(self.__set_time))
 
-        library.librarian.connect('removed', self.__set_time)
-        library.librarian.connect('added', self.__set_time)
-        library.librarian.connect_object('changed', self.__song_changed, player)
-        player.connect('song-ended', self.__song_ended)
-        player.connect('song-started', self.__song_started)
-        player.connect('paused', self.__update_paused, True)
-        player.connect('unpaused', self.__update_paused, False)
+        lib = library.librarian
+        gobject_weak(lib.connect, 'removed', self.__set_time, parent=self)
+        gobject_weak(lib.connect, 'added', self.__set_time, parent=self)
+        gobject_weak(lib.connect_object, 'changed', self.__song_changed,
+                     player, parent=self)
+
+        player_sigs = [
+            ('song-ended', self.__song_ended),
+            ('song-started', self.__song_started),
+            ('paused', self.__update_paused, True),
+            ('unpaused', self.__update_paused, False),
+        ]
+        for sig in player_sigs:
+            gobject_weak(player.connect, *sig, **{"parent": self})
 
         targets = [("text/uri-list", 0, 1)]
         self.drag_dest_set(
@@ -275,6 +289,15 @@ class QuodLibetWindow(gtk.Window):
         if config.getboolean('library', 'refresh_on_start'):
             self.__rebuild(None, False)
 
+        self.connect("delete-event", self.__save_browser)
+        self.connect("destroy", self.__destroy)
+
+    def __destroy(self, *args):
+        # The tray icon plugin tries to unhide QL because it gets disabled
+        # on Ql exit. The window should stay hidden after destroy.
+        self.show = lambda: None
+        self.present = self.show
+
     def __drag_motion(self, ctx, x, y, time):
         # Don't accept drops from QL itself, since it offers text/uri-list.
         if ctx.get_source_widget() is None:
@@ -292,7 +315,6 @@ class QuodLibetWindow(gtk.Window):
             uris = [uri.encode('ascii', 'replace')]
 
         dirs = []
-        files = []
         error = False
         for uri in uris:
             try: uri = URI(uri)
@@ -304,12 +326,10 @@ class QuodLibetWindow(gtk.Window):
                 else:
                     loc = os.path.realpath(loc)
                     if loc not in self.__library:
-                        song = self.__library.add_filename(loc)
-                        if song: files.append(song)
+                        self.__library.add_filename(loc)
             elif player.can_play_uri(uri):
                 if uri not in self.__library:
-                    files.append(RemoteFile(uri))
-                    self.__library.add([files[-1]])
+                    self.__library.add([RemoteFile(uri)])
             else:
                 error = True
                 break
@@ -330,22 +350,24 @@ class QuodLibetWindow(gtk.Window):
 
     def hide(self):
         super(QuodLibetWindow, self).hide()
-        map(gtk.Window.hide, qltk.Window.childs)
+        map(gtk.Window.hide, qltk.Window.instances)
 
     def present(self):
         super(QuodLibetWindow, self).present()
-        map(gtk.Window.present, qltk.Window.childs)
+        map(gtk.Window.present, qltk.Window.instances)
 
     def show(self):
         self.move(*map(int, config.get('memory', 'position').split()))
         super(QuodLibetWindow, self).show()
-        map(gtk.Window.show, qltk.Window.childs)
+        map(gtk.Window.show, qltk.Window.instances)
+
+    def __save_browser(self, *args):
+        print_d("Saving active browser state")
+        try: self.browser.save()
+        except NotImplementedError: pass
 
     def destroy(self, *args):
-        # The tray icon plugin tries to unhide QL because it gets disabled
-        # on Ql exit. The window should be hidden after destroy gets called.
-        self.show = lambda: None
-        self.present = self.show
+        self.__save_browser()
         super(QuodLibetWindow, self).destroy()
 
     def __show_or(self, widget, prop):
@@ -416,7 +438,7 @@ class QuodLibetWindow(gtk.Window):
              None, None, lambda *args: LoggingWindow(self)),
             ]
 
-        if "QUODLIBET_DEBUG" in os.environ:
+        if const.DEBUG:
             from quodlibet.debug import cause_error, enc
             actions.append(("DebugReload", gtk.STOCK_DIALOG_WARNING,
                             _("_Edit and Continue"), None, None,
@@ -436,7 +458,7 @@ class QuodLibetWindow(gtk.Window):
         ag.add_actions(actions)
 
         act = gtk.Action("About", None, None, gtk.STOCK_ABOUT)
-        act.connect_object('activate', about.show, self, player)
+        act.connect_object('activate', self.__show_about, player)
         ag.add_action_with_accel(act, None)
 
         act = gtk.Action(
@@ -494,11 +516,16 @@ class QuodLibetWindow(gtk.Window):
             act.connect_object('activate', LibraryBrowser, Kind, library)
             ag.add_action_with_accel(act, None)
 
+        debug_menu = ""
+        if const.DEBUG:
+            debug_menu = ("<menuitem action='DebugReload'/>\n"
+                          "<menuitem action='DebugCauseError'/>")
+
         self.ui = gtk.UIManager()
         self.ui.insert_action_group(ag, -1)
         menustr = const.MENU % {"browsers": browsers.BrowseLibrary(),
                                 "views": browsers.ViewBrowser(),
-                                "debug": const.DEBUG_MENU}
+                                "debug": debug_menu}
         self.ui.add_ui_from_string(menustr)
 
         # Cute. So. UIManager lets you attach tooltips, but when they're
@@ -519,6 +546,11 @@ class QuodLibetWindow(gtk.Window):
                 _("The 40 songs you've played least (more than 40 may "
                   "be chosen if there are ties)"))
 
+    def __show_about(self, player):
+        about = AboutQuodLibet(self, player)
+        about.run()
+        about.destroy()
+
     def __browser_configure(self, paned, event, browser):
         if paned.get_property('position-set'):
             key = "%s_pos" % browser.__class__.__name__
@@ -569,7 +601,7 @@ class QuodLibetWindow(gtk.Window):
             sig = container.connect('size-allocate', set_size, val)
             container._size_sig = sig
 
-        player.replaygain_profiles[0] = self.browser.replaygain_profiles
+        player.replaygain_profiles[1] = self.browser.replaygain_profiles
         player.volume = player.volume
         self.__vbox.pack_end(container)
         container.show()
@@ -579,7 +611,7 @@ class QuodLibetWindow(gtk.Window):
 
     def __update_paused(self, player, paused):
         menu = self.ui.get_widget("/Menu/Control/PlayPause")
-        if not menu or not menu.child: return # shutting down
+
         if paused: key = gtk.STOCK_MEDIA_PLAY
         else: key = gtk.STOCK_MEDIA_PAUSE
         text = gtk.stock_lookup(key)[1]
@@ -622,7 +654,8 @@ class QuodLibetWindow(gtk.Window):
                     "/Menu/Filters/Filter%s" % h.capitalize()).set_sensitive(
                     h in song)
 
-        if song and not self.songlist._activated and \
+        # don't jump on stream changes (player.info != player.song)
+        if song and player.song is song and not self.songlist._activated and \
             config.getboolean("settings", "jump"):
             self.__jump_to_current(False)
 
@@ -645,9 +678,12 @@ class QuodLibetWindow(gtk.Window):
         self.move(*pos)
 
     def __refresh_size(self):
+        ssv = self.song_scroller.get_property('visible')
+        qex = self.qexpander.get_property('visible')
+        brv = self.browser.expand
+
         self.__vbox.set_spacing(6)
-        if (not self.browser.expand and
-            not self.song_scroller.get_property('visible')):
+        if (not brv and not (ssv or qex)):
             if self.browser.size_request()[1] == 0:
                 self.__vbox.set_spacing(0)
             width, height = self.get_size()
@@ -690,9 +726,7 @@ class QuodLibetWindow(gtk.Window):
 
             self.songlist.scroll_to_cell(path, use_align=True, row_align=0.5)
             if select:
-                selection = self.songlist.get_selection()
-                selection.unselect_all()
-                selection.select_path(path)
+                self.songlist.set_cursor(path)
 
         song = player.playlist.song
         model =  self.songlist.model
@@ -808,13 +842,11 @@ class QuodLibetWindow(gtk.Window):
                 self.last_dir = fns[0]
                 copool.add(self.__library.scan, fns, funcid="library")
             else:
-                added = []
                 self.last_dir = os.path.basename(fns[0])
                 for filename in map(os.path.realpath, map(util.fsnative, fns)):
                     if filename in self.__library: continue
                     song = self.__library.add_filename(filename)
-                    if song: added.append(song)
-                    else:
+                    if not song:
                         from traceback import format_exception_only as feo
                         tb = feo(sys.last_type, sys.last_value)
                         msg = _("%s could not be added to your library.\n\n")
@@ -826,8 +858,6 @@ class QuodLibetWindow(gtk.Window):
                         d.label.set_selectable(True)
                         d.run()
                         continue
-                if added:
-                    self.browser.activate()
 
         if cb and cb.get_active():
             dirs = util.split_scan_dirs(config.get("settings", "scan"))
