@@ -21,9 +21,12 @@ from quodlibet.parse import Query, Pattern
 from quodlibet.qltk.information import Information
 from quodlibet.qltk.properties import SongProperties
 from quodlibet.qltk.views import AllTreeView
+from quodlibet.qltk.ratingsmenu import RatingsMenuItem
 from quodlibet.util.uri import URI
 from quodlibet.qltk.playorder import ORDERS
-from quodlibet.formats._audio import TAG_TO_SORT, AudioFile
+from quodlibet.formats._audio import TAG_TO_SORT, FILESYSTEM_TAGS, AudioFile
+from quodlibet.qltk.sortdialog import SortDialog
+from quodlibet.util import human_sort_key
 
 class PlaylistMux(object):
 
@@ -68,7 +71,7 @@ class PlaylistMux(object):
         self.pl.previous()
 
     def go_to(self, song, explicit=False):
-        print_d("Told to go to %r" % song, context=self)
+        print_d("Told to go to %r" % getattr(song, "key", song))
         self.q.go_to(None)
         return self.pl.go_to(song, explicit)
 
@@ -112,7 +115,7 @@ class PlaylistModel(gtk.ListStore):
         if oldsong is None: oldsong = self.__old_value
         else: self.__old_value = oldsong
         self.order.reset(self)
-        self.__iter = None
+        self.current_iter = None
 
         # We just reset the order manually so block the signals
         map(self.handler_block, self.__sigs)
@@ -123,7 +126,7 @@ class PlaylistModel(gtk.ListStore):
         for song in reversed(songs):
             iter = insert(0, (song,))
             if song is oldsong:
-                self.__iter = iter
+                self.current_iter = iter
         if self.__iter is not None:
             self.__old_value = None
         print_d("Done filling model.", context=self)
@@ -139,7 +142,7 @@ class PlaylistModel(gtk.ListStore):
 
     def remove(self, iter):
         if self.__iter and self[iter].path == self[self.__iter].path:
-            self.__iter = None
+            self.current_iter = None
         super(PlaylistModel, self).remove(iter)
 
     def get(self):
@@ -158,40 +161,48 @@ class PlaylistModel(gtk.ListStore):
         else: return self[self.__iter].path
     current_path = property(get_current_path)
 
+    def __set_current_iter(self, iter_):
+        # emit a row-changed for the previous and the new iter after it is set
+        # so that the currentcolumn icon gets updated on song changes
+        old_iter = self.current_iter
+        self.__iter = iter_
+        for iter_ in filter(None, (self.__iter, old_iter)):
+            self.row_changed(self.get_path(iter_), iter_)
+
     def get_current_iter(self):
         if self.__iter is None: return None
         elif self.is_empty(): return None
         else: return self.__iter
-    current_iter = property(get_current_iter)
+    current_iter = property(get_current_iter, __set_current_iter)
 
     def next(self):
-        self.__iter = self.order.next_explicit(self, self.__iter)
+        self.current_iter = self.order.next_explicit(self, self.__iter)
 
     def next_ended(self):
-        self.__iter = self.order.next_implicit(self, self.__iter)
+        self.current_iter = self.order.next_implicit(self, self.__iter)
 
     def previous(self):
-        self.__iter = self.order.previous_explicit(self, self.__iter)
+        self.current_iter = self.order.previous_explicit(self, self.__iter)
 
     def go_to(self, song, explicit=False):
-        print_d("Told to go to %r" % song, context=self)
-        self.__iter = None
+        print_d("Told to go to %r" % getattr(song, "key", song))
+        self.current_iter = None
         if isinstance(song, gtk.TreeIter):
-            self.__iter = song
+            self.current_iter = song
             self.sourced = True
-        else:
+        elif song is not None:
             for row in self:
                 if row[0] == song:
-                    self.__iter = row.iter
+                    self.current_iter = row.iter
                     print_d("Found song at %r" % row, context=self)
                     self.sourced = True
                     break
             else:
                 print_d("Failed to find song", context=self)
         if explicit:
-            self.__iter = self.order.set_explicit(self, self.__iter)
+            self.current_iter = self.order.set_explicit(self, self.__iter)
         else:
-            self.__iter = self.order.set_implicit(self, self.__iter)
+            self.current_iter = self.order.set_implicit(self, self.__iter)
         return self.__iter
 
     def find(self, song):
@@ -334,11 +345,10 @@ class SongList(AllTreeView, util.InstanceTracker):
     class FSColumn(WideTextColumn):
         # Contains text in the filesystem encoding, so needs to be
         # decoded safely (and also more slowly).
-        def _cdf(self, column, cell, model, iter, tag, code=const.FSCODING):
+        def _cdf(self, column, cell, model, iter, tag):
             value = model[iter][0].comma(tag)
             if not self._needs_update(value): return
-            cell.set_property('text', util.unexpand(
-                value.decode(code, 'replace')))
+            cell.set_property('text', util.unexpand(util.fsdecode(value)))
 
     class NumericColumn(TextColumn):
         # Any '~#' keys except dates.
@@ -364,9 +374,21 @@ class SongList(AllTreeView, util.InstanceTracker):
         def __init__(self):
             super(SongList.LengthColumn, self).__init__("~#length")
 
+    class FilesizeColumn(NumericColumn):
+        def _cdf(self, column, cell, model, iter, tag):
+            value = model[iter][0].get("~#filesize", 0)
+            if not self._needs_update(value): return
+            text = util.format_size(value)
+            cell.set_property('text', text)
+            self._update_layout(text, cell)
+        def __init__(self):
+            super(SongList.FilesizeColumn, self).__init__("~#filesize")
+
     class PatternColumn(WideTextColumn):
         def _cdf(self, column, cell, model, iter, tag):
             song = model.get_value(iter, 0)
+            if not self._pattern:
+                return
             value = self._pattern % song
             if not self._needs_update(value): return
             cell.set_property('text', value)
@@ -374,7 +396,9 @@ class SongList(AllTreeView, util.InstanceTracker):
         def __init__(self, pattern):
             super(SongList.PatternColumn, self).__init__(util.pattern(pattern))
             self.header_name = pattern
-            self._pattern = Pattern(pattern)
+            self._pattern = None
+            try: self._pattern = Pattern(pattern)
+            except ValueError: pass
 
     def Menu(self, header, browser, library):
         songs = self.get_selected_songs()
@@ -402,21 +426,13 @@ class SongList(AllTreeView, util.InstanceTracker):
         if (header not in ["artist", "album"] and can_filter(header)):
             menu.prepend(Filter(header))
 
-        item = gtk.MenuItem(_("_Rating"))
-        m2 = gtk.Menu()
-        item.set_submenu(m2)
-        for i in range(0, int(1.0/util.RATING_PRECISION)+1):
-            i *= util.RATING_PRECISION
-            itm = gtk.MenuItem("%0.2f\t%s" % (i, util.format_rating(i)))
-            m2.append(itm)
-            itm.connect_object(
-                'activate', self.__set_rating, i, songs, library)
+        ratings = RatingsMenuItem(songs, library)
         menu.preseparate()
-        menu.prepend(item)
+        menu.prepend(ratings)
         menu.show_all()
         return menu
 
-    def __init__(self, library, player=None):
+    def __init__(self, library, player=None, update=False):
         super(SongList, self).__init__()
         self._register_instance(SongList)
         self.set_model(PlaylistModel())
@@ -424,6 +440,7 @@ class SongList(AllTreeView, util.InstanceTracker):
         self.set_rules_hint(True)
         self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
         self.set_fixed_height_mode(True)
+        self.__csig = self.connect('columns-changed', self.__columns_changed)
         self.set_column_headers(self.headers)
         librarian = library.librarian or library
         sigs = []
@@ -434,6 +451,8 @@ class SongList(AllTreeView, util.InstanceTracker):
             sigs.append(s)
         sigs.extend([librarian.connect('changed', self.__song_updated),
                 librarian.connect('removed', self.__song_removed)])
+        if update:
+            sigs.append(librarian.connect('added', self.__song_added))
         for sig in sigs:
             self.connect_object('destroy', librarian.disconnect, sig)
         if player:
@@ -451,7 +470,6 @@ class SongList(AllTreeView, util.InstanceTracker):
         self.connect('drag-data-get', self.__drag_data_get)
         self.connect('drag-data-received', self.__drag_data_received, library)
 
-        # Enabling this screws up rating and enqueuing
         self.set_search_equal_func(self.__search_func)
 
         self.accelerators = gtk.AccelGroup()
@@ -462,15 +480,18 @@ class SongList(AllTreeView, util.InstanceTracker):
         self.accelerators.connect_group(
             key, mod, 0, lambda *args: self.__information(librarian))
 
+        self.connect('destroy', self.__destroy)
+
         self.__scroll_delay = None
         self.__scroll_periodic = None
         self.__scroll_args = (0, 0, 0, 0)
         self.__scroll_length = 0
         self.__scroll_last = None
 
-        # If the a song changes, we can't simply reverse the model
-        # (same tag, changed song order)
-        self.__sort_dirty = False
+    def __destroy(self, *args):
+        self.handler_block(self.__csig)
+        map(self.remove_column, self.get_columns())
+        self.handler_unblock(self.__csig)
 
     def __search_func(self, model, column, key, iter, *args):
         for column in self.get_columns():
@@ -720,6 +741,31 @@ class SongList(AllTreeView, util.InstanceTracker):
             for song in songs: values.update(song.list(header))
         browser.filter(header, list(values))
 
+    def __custom_sort(self, *args):
+        sd = SortDialog(qltk.get_top_parent(self))
+        if sd.run() == gtk.RESPONSE_OK:
+            # sort_keys yields a list of pairs (sort header, order)
+            headers = sd.sort_keys()
+            if not headers:
+                return
+            # from this, we have to construct a comparison function for sort
+            def _get_key(song, tag):
+                if tag.startswith("~#") and "~" not in tag[2:]:
+                    return song(tag)
+                return human_sort_key(song(tag))
+            def comparer(x, y):
+                for (h, o) in headers:
+                    c = cmp(_get_key(x, h), _get_key(y, h))
+                    if c == 0: continue
+                    if o != gtk.SORT_ASCENDING:
+                        c *= -1
+                    return c
+                return 0
+            songs = self.get_songs()
+            songs.sort(cmp=comparer)
+            self.set_songs(songs, sorted=True)
+        sd.hide()
+
     def __button_press(self, view, event, librarian):
         if event.button != 1: return
         x, y = map(int, [event.x, event.y])
@@ -753,16 +799,16 @@ class SongList(AllTreeView, util.InstanceTracker):
             song["~#rating"] = value
         librarian.changed(songs)
 
-    def __key_press(self, songlist, event, librarian,
-                    start_search=gtk.accelerator_parse("<control>F")):
+    def __key_press(self, songlist, event, librarian):
         if event.string in ['0', '1', '2', '3', '4']:
             rating = min(1.0, int(event.string) * util.RATING_PRECISION)
             self.__set_rating(rating, self.get_selected_songs(), librarian)
             return True
-        elif event.string in ['Q', 'q']:
+        elif qltk.is_accel(event, "<ctrl>Return") or \
+            qltk.is_accel(event, "<ctrl>KP_Enter"):
             self.__enqueue(self.get_selected_songs())
             return True
-        elif (event.keyval, event.state) == start_search:
+        elif qltk.is_accel(event, "<control>F"):
             self.emit('start-interactive-search')
             return True
         return False
@@ -774,8 +820,12 @@ class SongList(AllTreeView, util.InstanceTracker):
             main.playlist.enqueue(songs)
 
     def __redraw_current(self, player, song=None):
-        iter = self.model.current_iter
-        if iter: self.model.row_changed(self.model.get_path(iter), iter)
+        self.model.current_iter = self.model.current_iter
+
+    def __columns_changed(self, *args):
+        headers = map(lambda h: h.header_name, self.get_columns())
+        SongList.set_all_column_headers(headers)
+        SongList.headers = headers
 
     def set_all_column_headers(cls, headers):
         config.set("settings", "headers", " ".join(headers))
@@ -788,7 +838,10 @@ class SongList(AllTreeView, util.InstanceTracker):
         star = list(Query.STAR)
         for header in headers:
             if "<" in header:
-                tags = Pattern(header).tags
+                try:
+                    tags = Pattern(header).tags
+                except ValueError:
+                    continue
             else:
                 tags = util.tagsplit(header)
             for tag in tags:
@@ -804,7 +857,7 @@ class SongList(AllTreeView, util.InstanceTracker):
                 tag = header.header_name
                 sort = header.get_sort_order()
                 return (tag, sort == gtk.SORT_DESCENDING)
-        else: return "album", False
+        else: return "", False
 
     def is_sorted(self):
         return max([c.get_sort_indicator() for c in self.get_columns()] or [0])
@@ -830,15 +883,21 @@ class SongList(AllTreeView, util.InstanceTracker):
                 h.set_sort_order(s)
             else: h.set_sort_indicator(False)
         if refresh:
-            if rev:
-                if self.__sort_dirty:
-                    # python sort is faster if it's presorted.
-                    songs = self.get_songs()
-                    songs.reverse()
-                    self.set_songs(songs)
-                else:
-                    self.reverse()
-            else: self.set_songs(self.get_songs())
+            songs = self.get_songs()
+            if rev:  # python sort is faster if it's presorted.
+                songs.reverse()
+            self.set_songs(songs)
+
+    def set_sort_by_tag(self, tag, order=None):
+        for h in self.get_columns():
+            name = h.header_name
+            if self.__get_sort_tag(name) == tag:
+                if order: s = gtk.SORT_DESCENDING
+                else: s = gtk.SORT_ASCENDING
+                h.set_sort_order(s)
+                h.set_sort_indicator(True)
+            else:
+                h.set_sort_indicator(False)
 
     def set_model(self, model):
         super(SongList, self).set_model(model)
@@ -851,8 +910,8 @@ class SongList(AllTreeView, util.InstanceTracker):
 
     def __get_sort_tag(self, tag):
         replace_order = {
-            "~#track": "album",
-            "~#disc": "album",
+            "~#track": "",
+            "~#disc": "",
             "~length": "~#length"
         }
 
@@ -878,9 +937,39 @@ class SongList(AllTreeView, util.InstanceTracker):
 
         return tag
 
+    def add_songs(self, songs):
+        """Add songs to the list in the right order and position"""
+
+        if not songs: return
+
+        model = self.get_model()
+        if not len(model):
+            self.set_songs(songs)
+            return
+
+        tag, reverse = self.get_sort_by()
+        tag = self.__get_sort_tag(tag)
+
+        if not self.is_sorted():
+            self.set_sort_by_tag(tag, reverse)
+
+        # FIXME: Replace with something fast
+
+        old_songs = self.get_songs()
+        old_songs.extend(songs)
+
+        if not tag:
+            old_songs.sort(key=lambda s: s.sort_key, reverse=reverse)
+        else:
+            sort_func = AudioFile.sort_by_func(tag)
+            old_songs.sort(key=lambda s: s.sort_key)
+            old_songs.sort(key=sort_func, reverse=reverse)
+
+        for index, song in sorted(zip(map(old_songs.index, songs), songs)):
+            model.insert(index, row=[song])
+
     def set_songs(self, songs, sorted=False):
         model = self.get_model()
-        self.__sort_dirty = False
 
         if not sorted:
             tag, reverse = self.get_sort_by()
@@ -888,15 +977,14 @@ class SongList(AllTreeView, util.InstanceTracker):
 
             #try to set a sort indicator that matches the default order
             if not self.is_sorted():
-                for h in self.get_columns():
-                    name = h.header_name
-                    if self.__get_sort_tag(name) == tag:
-                        h.set_sort_indicator(True)
-                        h.set_sort_order(gtk.SORT_ASCENDING)
-                        break
+                self.set_sort_by_tag(tag, reverse)
 
-            sort_func = AudioFile.sort_by_func(tag)
-            songs.sort(key=sort_func, reverse=reverse)
+            if not tag:
+                songs.sort(key=lambda s: s.sort_key, reverse=reverse)
+            else:
+                sort_func = AudioFile.sort_by_func(tag)
+                songs.sort(key=lambda s: s.sort_key)
+                songs.sort(key=sort_func, reverse=reverse)
         else:
             self.set_sort_by(None, refresh=False)
 
@@ -911,14 +999,6 @@ class SongList(AllTreeView, util.InstanceTracker):
         print_d("Model attached.", context=self)
         map(gtk.TreeViewColumn.set_sort_indicator, self.get_columns(), sorts)
 
-    def reverse(self):
-        model = self.get_model()
-        sorts = map(gtk.TreeViewColumn.get_sort_indicator, self.get_columns())
-        self.set_model(None)
-        model.reverse()
-        self.set_model(model)
-        map(gtk.TreeViewColumn.set_sort_indicator, self.get_columns(), sorts)
-
     def get_selected_songs(self):
         selection = self.get_selection()
         if selection is None: return []
@@ -928,7 +1008,6 @@ class SongList(AllTreeView, util.InstanceTracker):
     def __song_updated(self, librarian, songs):
         """Only update rows that are currently displayed.
         Warning: This makes the row-changed signal useless."""
-        self.__sort_dirty = True
         #pygtk 2.12: prevent invalid ranges or GTK asserts
         if not self.flags() & gtk.REALIZED or \
             self.get_path_at_pos(0,0) is None: return
@@ -941,10 +1020,51 @@ class SongList(AllTreeView, util.InstanceTracker):
             if row[0] in songs:
                 model.row_changed(row.path, row.iter)
 
+    def __song_added(self, librarian, songs):
+        window = qltk.get_top_parent(self)
+        filter_ = window.browser.active_filter
+        if callable(filter_):
+            self.add_songs(filter(filter_, songs))
+
     def __song_removed(self, librarian, songs):
         # The selected songs are removed from the library and should
         # be removed from the view.
-        map(self.model.remove, self.model.find_all(songs))
+
+        if not len(self.model):
+            return
+
+        songs = set(songs)
+
+        # search in the selection first
+        # speeds up common case: select songs and remove them
+        model, rows = self.get_selection().get_selected_rows()
+        rows = rows or []
+        iters = [model[r].iter for r in rows if model[r][0] in songs]
+
+        # if not all songs were in the selection, search the whole view
+        if len(iters) != len(songs):
+            iters = model.find_all(songs)
+
+        selection_removed = (len(iters) == len(rows))
+        self._remove(iters, set_selection=selection_removed)
+
+    def _remove(self, iters, set_selection=True):
+        """Removes the rows from the model and selects the row
+        after the last removed one if specified."""
+
+        model = self.model
+        map(model.remove, iters)
+
+        if not set_selection: return
+
+        # model.remove makes the removed iter point to the next row if possible
+        # so check if the last iter is a valid one and select it or
+        # simply select the last row
+        selection = self.get_selection()
+        if len(iters) and model.iter_is_valid(iters[-1]):
+            selection.select_iter(iters[-1])
+        elif len(model):
+            selection.select_path(model[-1].path)
 
     def __song_properties(self, librarian):
         model, rows = self.get_selection().get_selected_rows()
@@ -973,6 +1093,8 @@ class SongList(AllTreeView, util.InstanceTracker):
     def set_column_headers(self, headers):
         if len(headers) == 0: return
 
+        self.handler_block(self.__csig)
+
         old_sort = self.is_sorted() and self.get_sort_by()
         map(self.remove_column, self.get_columns())
 
@@ -985,9 +1107,10 @@ class SongList(AllTreeView, util.InstanceTracker):
             elif t in ["~#added", "~#mtime", "~#lastplayed", "~#laststarted"]:
                 column = self.DateColumn(t)
             elif t in ["~length", "~#length"]: column = self.LengthColumn()
+            elif t == "~#filesize": column = self.FilesizeColumn()
             elif t in ["~rating", "~#rating"]: column = self.RatingColumn()
             elif t.startswith("~#"): column = self.NumericColumn(t)
-            elif t in ["~filename", "~basename", "~dirname"]:
+            elif t in FILESYSTEM_TAGS:
                 column = self.FSColumn(t)
             elif t.startswith("<"):
                 column = self.PatternColumn(t)
@@ -1004,6 +1127,8 @@ class SongList(AllTreeView, util.InstanceTracker):
             header, order = old_sort
             self.set_sort_by(None, header, order, False)
 
+        self.handler_unblock(self.__csig)
+
     def __getmenu(self, column):
         menu = gtk.Menu()
         menu.connect_object('selection-done', gtk.Menu.destroy, menu)
@@ -1040,12 +1165,15 @@ class SongList(AllTreeView, util.InstanceTracker):
             ~#tracks albumartist""".split()
         dateinfo = """date originaldate recordingdate ~#laststarted
             ~#lastplayed ~#added ~#mtime""".split()
-        fileinfo = """~format ~#bitrate ~filename ~basename ~dirname
+        fileinfo = """~format ~#bitrate ~#filesize ~filename ~basename ~dirname
             ~uri""".split()
         copyinfo = """copyright organization location isrc
             contact website""".split()
+        all_headers = reduce(lambda x,y: x+y,
+            [trackinfo, peopleinfo, albuminfo, dateinfo, fileinfo, copyinfo])
 
         for name, group in [
+            (_("All _Headers"), all_headers),
             (_("_Track Headers"), trackinfo),
             (_("_Album Headers"), albuminfo),
             (_("_People Headers"), peopleinfo),
@@ -1064,6 +1192,12 @@ class SongList(AllTreeView, util.InstanceTracker):
         sep = gtk.SeparatorMenuItem()
         sep.show()
         menu.append(sep)
+
+        b = gtk.MenuItem(_("Custom _Sort..."))
+        menu.append(b)
+        b.show()
+        b.connect('activate', self.__custom_sort)
+
         custom = gtk.MenuItem(_("_Customize Headers..."))
         custom.show()
         custom.connect('activate', self.__add_custom_column)
@@ -1087,8 +1221,7 @@ class SongList(AllTreeView, util.InstanceTracker):
         # Prefs has to import SongList, so do this here to avoid
         # a circular import.
         from quodlibet.qltk.prefs import PreferencesWindow
-        win = PreferencesWindow(self)
-        win.child.set_current_page(0)
+        PreferencesWindow(self).set_page("songlist")
 
     def __showmenu(self, column, event=None):
         time = gtk.get_current_event_time()
