@@ -81,6 +81,7 @@ class OptionParser(object):
             "help", shorts="h", help=_("Display brief usage information"))
         self.add(
             "version", shorts="v", help=_("Display version and copyright"))
+        self.add("debug", shorts="d")
 
     def add(self, canon, help=None, arg="", shorts="", longs=[]):
         self.__args[canon] = arg
@@ -179,6 +180,9 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
                 elif o == "version":
                     print_(self.version())
                     raise SystemExit
+                elif o == "debug":
+                    from quodlibet import const
+                    const.DEBUG = True
                 if self.__args[o]: transopts[o] = a
                 else: transopts[o] = True
 
@@ -242,19 +246,19 @@ def format_size(size):
     """Turn an integer size value into something human-readable."""
     # TODO: Better i18n of this (eg use O/KO/MO/GO in French)
     if size >= 1024*1024*1024:
-        return "%.1fGB" % (float(size) / (1024*1024*1024))
+        return "%.1f GB" % (float(size) / (1024*1024*1024))
     elif size >= 1024*1024 * 100:
-        return "%.0fMB" % (float(size) / (1024*1024))
+        return "%.0f MB" % (float(size) / (1024*1024))
     elif size >= 1024*1024 * 10:
-        return "%.1fMB" % (float(size) / (1024*1024))
+        return "%.1f MB" % (float(size) / (1024*1024))
     elif size >= 1024*1024:
-        return "%.2fMB" % (float(size) / (1024*1024))
+        return "%.2f MB" % (float(size) / (1024*1024))
     elif size >= 1024 * 10:
-        return "%dKB" % int(size / 1024)
+        return "%d KB" % int(size / 1024)
     elif size >= 1024:
-        return "%.2fKB" % (float(size) / 1024)
+        return "%.2f KB" % (float(size) / 1024)
     else:
-        return "%dB" % size
+        return "%d B" % size
 
 def format_time(time):
     """Turn a time value in seconds into hh:mm:ss or mm:ss."""
@@ -290,16 +294,23 @@ def format_time_long(time):
     if len(time_str) > 2: time_str.pop()
     return ", ".join(time_str)
 
-def fsdecode(s):
-    """Decoding a string according to the filesystem encoding."""
+def fsdecode(s, note=True):
+    """Decoding a string according to the filesystem encoding.
+    note specifies whether a note should be appended if decoding failed."""
     if isinstance(s, unicode): return s
-    else: return decode(s, fscoding)
+    elif note:
+        return decode(s, fscoding)
+    else:
+        return s.decode(fscoding, 'replace')
 
-def fsencode(s):
-    """Encode a string according to the filesystem encoding, replacing
-    errors."""
+def fsencode(s, note=False):
+    """Encode a string according to the filesystem encoding.
+    note specifies whether a note should be appended if encoding failed."""
     if isinstance(s, str): return s
-    else: return s.encode(fscoding, 'replace')
+    elif note:
+        return encode(s, fscoding)
+    else:
+        return s.encode(fscoding, 'replace')
 
 """
 Path related functions like open, os.listdir have different behavior on win32
@@ -379,14 +390,17 @@ def split_title(s, splitters=["/", "&", ","]):
     if not subtitle: return (s, [])
     else: return (title.strip(), split_value(subtitle, splitters))
 
-def split_people(s, splitters=["/", "&", ","]):
-    FEATURING = ["feat.", "featuring", "feat", "ft", "ft.", "with", "w/"]
 
+__FEATURING = ["feat.", "featuring", "feat", "ft", "ft.", "with", "w/"]
+# Cache case-insensitive regex searches of the above
+__FEAT_REGEX = [re.compile(re.escape(s + " "), re.I) for s in __FEATURING]
+
+def split_people(s, splitters=["/", "&", ","]):
     title, subtitle = find_subtitle(s)
     if not subtitle:
         parts = s.split(" ")
         if len(parts) > 2:
-            for feat in FEATURING:
+            for feat in __FEATURING:
                 try:
                     i = [p.lower() for p in parts].index(feat)
                     orig = " ".join(parts[:i])
@@ -395,9 +409,11 @@ def split_people(s, splitters=["/", "&", ","]):
                 except (ValueError, IndexError): pass
         return (s, [])
     else:
-        for feat in FEATURING:
-            if subtitle.startswith(feat):
-                subtitle = subtitle.replace(feat, "", 1).lstrip()
+        old = subtitle
+        for regex in __FEAT_REGEX:
+            subtitle = re.sub(regex, "", subtitle, 1)
+            if old != subtitle:
+                # only change once
                 break
         values = split_value(subtitle, splitters)
         return (title.strip(), values)
@@ -419,22 +435,26 @@ def split_album(s):
             except: return (s, None)
         else: return (s, None)
 
-def split_numeric(s, reg=re.compile(r"([0-9]+\.?[0-9]*)")):
+def split_numeric(s, limit=10,
+        reg=re.compile(r"[0-9][0-9]*\.?[0-9]*").search,
+        join=u" ".join):
     """Seperate numeric values from the string and convert to float, so
     it can be used for human sorting. Also removes all extra whitespace."""
-    if not s: return ('',)
-    result = reg.search(s)
-    if not result: return (u" ".join(s.split()),)
+    result = reg(s)
+    if not result or not limit:
+        return (join(s.split()),)
     else:
-        first = u" ".join(s[:result.start()].split())
-        last = s[result.end():].strip()
-        return (first, float(result.group(0)), split_numeric(last))
+        start, end = result.span()
+        return (
+            join(s[:start].split()),
+            float(result.group()),
+            split_numeric(s[end:], limit - 1))
 
-def human_sort_key(s):
+def human_sort_key(s, normalize=unicodedata.normalize):
     if not isinstance(s, unicode):
         s = s.decode("utf-8")
-    s = unicodedata.normalize('NFD', s.lower())
-    return split_numeric(s[:1024])
+    s = normalize("NFD", s.lower())
+    return s and split_numeric(s)
 
 def find_subtitle(title):
     if isinstance(title, str): title = title.decode('utf-8', 'replace')
@@ -532,7 +552,11 @@ def pattern(pat, cap=True, esc=False):
 
     fakesong = Fakesong({'filename': tag('filename', cap)})
     fakesong.cap = cap
-    p = (esc and XMLFromPattern(pat)) or Pattern(pat)
+    try:
+        p = (esc and XMLFromPattern(pat)) or Pattern(pat)
+    except ValueError:
+        return _("Invalid pattern")
+
     return p.format(fakesong)
 
 def spawn(argv, stdout=False):
@@ -663,3 +687,91 @@ if os.name == "nt":
     pathname2url = pathname2url_win32
 else:
     pathname2url = urllib.pathname2url
+
+
+# See http://stackoverflow.com/questions/1151658/python-hashable-dicts
+class HashableDict(dict):
+    """Standard dict, made hashable. Useful for making sets of dicts etc"""
+    def __key(self):
+        return tuple((k,self[k]) for k in sorted(self))
+    def __hash__(self):
+        return hash(self.__key())
+    def __eq__(self, other):
+        return self.__key() == other.__key()
+
+
+def sanitize_tags(tags, stream=False):
+    """Returns a new sanitized tag dict. stream defines if the
+    tags of a main/base song should be changed or of a stream song.
+    e.g. title will be removed for the base song but not for the stream one.
+    """
+
+    san = {}
+    for key, value in tags.iteritems():
+        key = key.lower()
+        key = {"location": "website"}.get(key, key)
+
+        if isinstance(value, unicode):
+            lower = value.lower().strip()
+
+            if key == "channel-mode":
+                if "stereo" in lower or "dual" in lower:
+                    value = u"stereo"
+            elif key == "audio-codec":
+                if "mp3" in lower:
+                    value = u"MP3"
+                elif "aac" in lower or "advanced" in lower:
+                    value = u"MPEG-4 AAC"
+                elif "vorbis" in lower:
+                    value = u"Ogg Vorbis"
+
+            if lower in ("http://www.shoutcast.com", "http://localhost/",
+                "default genre", "none", "http://", "unnamed server",
+                "unspecified", "n/a"):
+                continue
+
+        if key == "duration":
+            try: value = int(long(value) / 1000)
+            except ValueError: pass
+            else:
+                if not stream: continue
+                key = "~#length"
+        elif key == "bitrate":
+            try: value = int(value) / 1000
+            except ValueError: pass
+            else:
+                if not stream: continue
+                key = "~#bitrate"
+        elif key == "nominal-bitrate":
+            try: value = int(value) / 1000
+            except ValueError: pass
+            else:
+                if stream: continue
+                key = "~#bitrate"
+
+        if key in ("emphasis", "mode", "layer", "maximum-bitrate",
+            "minimum-bitrate", "has-crc", "homepage"):
+            continue
+
+        if not stream and key in ("title", "album", "artist", "date"):
+            continue
+
+        if isinstance(value, (int, long, float)):
+            if not key.startswith("~#"):
+                key = "~#" + key
+            san[key] = value
+        else:
+            if key.startswith("~#"):
+                key = key[2:]
+
+            if not isinstance(value, unicode):
+                continue
+
+            value = value.strip()
+            if key in san:
+                if value not in san[key].split("\n"):
+                    san[key] += "\n" + value
+            else:
+                san[key] = value
+
+    return san
