@@ -3,7 +3,7 @@ from tests import TestCase, add
 import os
 
 from quodlibet import config
-
+from quodlibet.util import HashableDict
 from quodlibet.formats._audio import AudioFile
 from quodlibet.formats._audio import INTERN_NUM_DEFAULT
 
@@ -230,12 +230,20 @@ class TAudioFile(TestCase):
         for key in INTERN_NUM_DEFAULT:
             self.failUnless(key in dump)
 
+        n = AudioFile()
+        n.from_dump(dump)
+        self.failUnless(set(dump.split("\n")) == set(n.to_dump().split("\n")))
+
     def test_to_dump_long(self):
         b = AudioFile(bar_1_1); b["~#length"] = 200000000000L
         dump = b.to_dump()
         num = len(set(bar_1_1.keys()) | INTERN_NUM_DEFAULT)
         self.failUnlessEqual(dump.count("\n"), num + 2)
 
+        n = AudioFile()
+        n.from_dump(dump)
+        self.failUnless(set(dump.split("\n")) == set(n.to_dump().split("\n")))
+
     def test_add(self):
         song = AudioFile()
         self.failIf("foo" in song)
@@ -314,20 +322,109 @@ class TAudioFile(TestCase):
 
     def test_album_key(self):
         album_key_tests = [
-            ({}, (('',), '')),
+            ({}, ('', '')),
             ({'album': 'foo'}, (('foo',), '')),
-            ({'labelid': 'foo'}, (('',), 'foo')),
-            ({'musicbrainz_albumid': 'foo'}, (('',), 'foo')),
+            ({'labelid': 'foo'}, ('', 'foo')),
+            ({'musicbrainz_albumid': 'foo'}, ('', 'foo')),
             ({'album': 'foo', 'labelid': 'bar'}, (('foo',), 'bar')),
             ({'album': 'foo', 'labelid': 'bar', 'musicbrainz_albumid': 'quux'},
                 (('foo',), 'bar'))
             ]
-
         for tags, expected in album_key_tests:
             afile = AudioFile(**tags)
             afile.sanitize('/dir/fn')
             self.failUnlessEqual(afile.album_key, expected)
 
+    def test_album_id_values(self):
+        album_key_tests = [
+            ({}, {}),
+            ({'album': 'foo'}, None),
+            ({'labelid': 'foo'}, None),
+            ({'musicbrainz_albumid': 'foo'}, None),
+            ({'album': 'foo', 'labelid': 'bar'}, {'labelid': 'bar'}),
+            # Musicbrainz is our current "favourite" tag
+            ({'album': 'foo', 'labelid': 'bar', 'musicbrainz_albumid': 'quux'},
+                    {'musicbrainz_albumid': 'quux'}),
+            # albumartist > album in this case
+            ({'album': 'foo', 'albumartist': 'bar', 'artist': 'baz'},
+                    {'album': 'foo', 'albumartist': 'bar'}),
+            # Fail as there's nothing to specify the album
+            ({'albumartist': 'bar', 'artist': 'baz', 'title': 'foo'}, {})
+            ]
+        for tags, expected in album_key_tests:
+            if expected is None: expected = tags
+            afile = AudioFile(**tags)
+            afile.sanitize('/dir/fn')
+            self.failUnlessEqual(afile.album_id_values, HashableDict(expected))
+
+    def test_album_id_values_with_artist(self):
+        album_key_tests = [
+            # The artist should now appear
+            ({'album': 'foo', 'artist': 'bar', 'title': 'baz'},
+                    {'album': 'foo', 'artist': 'bar'}),
+            # But albumartist > album still
+            ({'album': 'foo', 'albumartist': 'bar', 'artist': 'baz'},
+                    {'album': 'foo', 'albumartist': 'bar'}),
+            ]
+        for tags, expected in album_key_tests:
+            if expected is None: expected = tags
+            afile = AudioFile(**tags)
+            afile.sanitize('/dir/fn')
+            # Use alternate version, that includes artist in dict
+            self.failUnlessEqual(afile._album_id_values(True),
+                                 HashableDict(expected))
+
+    def test_eq_ne(self):
+        self.failIf(AudioFile({"a": "b"}) == AudioFile({"a": "b"}))
+        self.failUnless(AudioFile({"a": "b"}) != AudioFile({"a": "b"}))
+
+    def test_invalid_fs_encoding(self):
+        # issue 798
+        a = AudioFile()
+        a.sanitize("/\xf6\xe4\xfc/\xf6\xe4\xfc.ogg") # latin 1 encoded
+        a.sort_by_func("~filename")(a)
+        a.sort_by_func("~basename")(a)
+        a.sort_by_func("~dirname")(a)
+
+        # windows
+        a.sanitize("/\xf6\xe4\xfc/\xf6\xe4\xfc.ogg".decode("latin-1"))
+        a.sort_by_func("~filename")(a)
+        a.sort_by_func("~basename")(a)
+        a.sort_by_func("~dirname")(a)
+
+    def test_sort_cache(self):
+        copy = AudioFile(bar_1_1)
+
+        sort_1 = tuple(copy.sort_key)
+        copy["title"] = copy["title"] + "something"
+        sort_2 = tuple(copy.sort_key)
+        self.failIfEqual(sort_1, sort_2)
+
+        album_sort_1 = tuple(copy.album_key)
+        copy["album"] = copy["album"] + "something"
+        sort_3 = tuple(copy.sort_key)
+        self.failIfEqual(sort_2, sort_3)
+
+        album_sort_2 = tuple(copy.album_key)
+        self.failIfEqual(album_sort_1, album_sort_2)
+
+    def test_cache_attributes(self):
+        x = AudioFile()
+        x.multisong = not x.multisong
+        x["a"] = "b" # clears cache
+        # attribute should be unchanged
+        self.failIfEqual(AudioFile().multisong, x.multisong)
+
+    def test_sort_func(self):
+        tags = [lambda s: s("foo"), "artistsort", "albumsort",
+                "~filename", "~format", "discnumber", "~#track"]
+
+        for tag in tags:
+            f = AudioFile.sort_by_func(tag)
+            f(bar_1_1)
+            f(bar_1_2)
+            f(bar_2_1)
+
     def tearDown(self):
         os.unlink(quux["~filename"])
 add(TAudioFile)
@@ -338,10 +435,11 @@ class Treplay_gain(TestCase):
     minus_6db = 0.501187234
 
     def setUp(self):
-        self.song = AudioFile({"replaygain_album_gain": "-1.00 dB",
-                               "replaygain_album_peak": "1.1",
-                               "replaygain_track_gain": "+1.0 dB",
-                               "replaygain_track_peak": "0.9"})
+        self.rg_data = {"replaygain_album_gain": "-1.00 dB",
+                        "replaygain_album_peak": "1.1",
+                        "replaygain_track_gain": "+1.0000001 dB",
+                        "replaygain_track_peak": "0.9"}
+        self.song = AudioFile(self.rg_data)
 
     def test_nogain(self):
         self.failUnlessEqual(self.song.replay_gain(["none", "track"]), 1)
@@ -390,18 +488,37 @@ class Treplay_gain(TestCase):
         # verify defaulting to track when album is present
         self.failUnlessAlmostEqual(
             self.song.replay_gain(["album", "track"]), radio_rg)
+
+    def test_numeric_rg_tags(self):
+        """"Tests fully-numeric (ie no "db") RG tags.  See Issue 865"""
+        self.failUnless(self.song("replaygain_album_gain"), "-1.00 db")
+        for key, exp in self.rg_data.items():
+            # Hack the nasties off and produce the "real" expected value
+            exp = float(exp.split(" ")[0])
+            # Compare as floats. Seems fairer.
+            album_rg = self.song("~#%s" % key)
+            try:
+                val = float(album_rg)
+            except ValueError: self.fail("Invalid %s returned: %s" %
+                                         (key,album_rg))
+            self.failUnlessAlmostEqual(val, exp, places=5,
+                                 msg="%s should be %s not %s" % (key,exp,val))
 add(Treplay_gain)
 
 # Special test case for find_cover since it has to create/remove
 # various files.
 class Tfind_cover(TestCase):
     def setUp(self):
+        config.init()
         self.dir = os.path.realpath(quux("~dirname"))
-        self.files = [os.path.join(self.dir, "12345.jpg"),
-                      os.path.join(self.dir, "nothing.jpg")
+        self.files = [self.full_path("12345.jpg"),
+                      self.full_path("nothing.jpg")
                       ]
         for f in self.files: file(f, "w").close()
 
+    def full_path(self, path):
+        return os.path.join(self.dir, path)
+
     def test_dir_not_exist(self):
         self.failIf(bar_2_1.find_cover())
 
@@ -411,7 +528,7 @@ class Tfind_cover(TestCase):
     def test_labelid(self):
         quux["labelid"] = "12345"
         self.failUnlessEqual(os.path.abspath(quux.find_cover().name),
-                             os.path.join(self.dir, "12345.jpg"))
+                             self.full_path("12345.jpg"))
         del(quux["labelid"])
 
     def test_regular(self):
@@ -424,6 +541,25 @@ class Tfind_cover(TestCase):
             self.failUnlessEqual(os.path.abspath(quux.find_cover().name), f)
         self.test_labelid() # labelid must work with other files present
 
+    def test_intelligent(self):
+        song = quux
+        song["artist"] = "Q-Man"
+        song["title"] = "First Q falls hardest"
+        files = [self.full_path(f) for f in
+                 ["Quuxly - back.jpg", "Quuxly.jpg", "q-man - quxxly.jpg",
+                  "folder.jpeg", "Q-man - Quuxly (FRONT).jpg"]]
+        for f in files:
+            file(f, "w").close()
+            self.files.append(f)
+            cover = song.find_cover()
+            if cover:
+                actual = os.path.abspath(cover.name)
+                self.failUnlessEqual(actual, f)
+            else:
+                # Here, no cover is better than the back...
+                self.failUnlessEqual(f, self.full_path("Quuxly - back.jpg"))
+
     def tearDown(self):
         map(os.unlink, self.files)
+        config.quit()
 add(Tfind_cover)
