@@ -1,13 +1,18 @@
+import gtk
+
 import os
-import sys
 import unittest
 
 from tempfile import mkstemp
 
 from quodlibet.library.songs import SongLibrary, SongFileLibrary, SongLibrarian
 
-from tests import TestCase, add
+from tests import add
 from tests.test_library__library import Fake, TLibrary, TLibrarian
+from quodlibet.formats.xiph import FLACFile
+
+import shutil
+from quodlibet import config
 
 class FakeSong(Fake):
     def list(self, tag):
@@ -30,6 +35,7 @@ class TSongLibrary(TLibrary):
         song = FakeSong(10)
         self.library.add([song])
         self.library.rename(song, 20)
+        while gtk.events_pending(): gtk.main_iteration()
         self.failUnless(song in self.changed)
         self.failUnless(song in self.library)
         self.failUnless(song.key in self.library)
@@ -118,6 +124,44 @@ class TSongFileLibrary(TSongLibrary):
         self.failIf(new._valid)
         self.failIf(new in self.library)
         self.failUnlessEqual(new, self.library._masked[new][new])
+
+    def __get_file(self):
+        fd, filename = mkstemp(".flac")
+        shutil.copy(os.path.join('tests', 'data', 'empty.flac'), filename)
+        return filename
+
+    def test_add_filename(self):
+        config.init()
+        try:
+            filename = self.__get_file()
+            ret = self.library.add_filename(filename)
+            self.failUnless(ret)
+            self.failUnlessEqual(1, len(self.library))
+            self.failUnlessEqual(len(self.added), 1)
+            ret = self.library.add_filename(filename)
+            self.failUnless(ret)
+            self.failUnlessEqual(len(self.added), 1)
+            os.unlink(filename)
+
+            filename = self.__get_file()
+            ret = self.library.add_filename(filename, add=False)
+            self.failUnless(ret)
+            self.failIf(ret in self.library)
+            self.failUnlessEqual(len(self.added), 1)
+            self.library.add([ret])
+            self.failUnless(ret in self.library)
+            self.failUnlessEqual(len(self.added), 2)
+            self.failUnlessEqual(2, len(self.library))
+            os.unlink(filename)
+
+            ret = self.library.add_filename("")
+            self.failIf(ret)
+            self.failUnlessEqual(len(self.added), 2)
+            self.failUnlessEqual(2, len(self.library))
+
+        finally:
+            config.quit()
+
 add(TSongFileLibrary)
 
 class TSongLibrarian(TLibrarian):
@@ -140,6 +184,7 @@ class TSongLibrarian(TLibrarian):
         self.lib1.add([new])
         self.lib2.add([new])
         self.librarian.rename(new, 20)
+        while gtk.events_pending(): gtk.main_iteration()
         self.failUnlessEqual(new.key, 20)
         self.failUnless(new in self.lib1)
         self.failUnless(new in self.lib2)
