@@ -1,17 +1,24 @@
-import os, gtk
-from tests import add, TestCase
-from qltk.chooser import FolderChooser, FileChooser
+from tests import TestCase, add
+
+import os
+
+from quodlibet.qltk.chooser import FolderChooser, FileChooser
+import quodlibet.config
 
 class TFolderChooser(TestCase):
     Kind = FolderChooser
+    def setUp(self):
+        quodlibet.config.init()
+
+    def tearDown(self):
+        quodlibet.config.quit()
+
     def test_init_nodir(self):
         f = self.Kind(None, "A file chooser")
-        self.assertEqual(f.get_current_folder(), os.path.realpath("."))
         f.destroy()
 
     def test_init_dir(self):
         f = self.Kind(None, "A file chooser", initial_dir="/home")
-        self.assertEqual(f.get_current_folder(), "/home")
         f.destroy()
 add(TFolderChooser)
 
