@@ -1,5 +1,6 @@
-from tests import add, TestCase
-from qltk.getstring import GetStringDialog
+from tests import TestCase, add
+
+from quodlibet.qltk.getstring import GetStringDialog
 
 class TGetStringDialog(TestCase):
     def setUp(self):
