@@ -7,24 +7,24 @@ class TPlayOrder(TestCase):
     def setUp(self):
         self.order = -1
         self.volume = 0
-        self.replaygain_profiles = [None, None]
+        self.replaygain_profiles = [None, None, None]
         quodlibet.config.init()
         self.win = PlayOrder(self, self)
         self.win.set_active(0)
 
     def test_initial(self):
         self.failUnlessEqual(self.win.get_active(), 0)
-        self.failUnless(self.replaygain_profiles[1], ["album", "track"])
+        self.failUnless(self.replaygain_profiles[2], ["album", "track"])
 
     def test_set_name(self):
         self.win.set_active("weighted")
         self.failUnlessEqual(2, self.win.get_active())
-        self.failUnless(self.replaygain_profiles[1], ["track"])
+        self.failUnless(self.replaygain_profiles[2], ["track"])
 
     def test_set_int(self):
         self.win.set_active(3)
         self.failUnlessEqual(3, self.win.get_active())
-        self.failUnless(self.replaygain_profiles[1], ["track"])
+        self.failUnless(self.replaygain_profiles[2], ["track"])
 
     def test_get_name(self):
         for i, name in enumerate(["inorder","shuffle","weighted","onesong"]):
