@@ -1,14 +1,18 @@
-import os, gtk
-import const
 from tests import add, TestCase
-from qltk.remote import FSInterface
-from qltk.watcher import SongWatcher
+
+import os
+import gtk
+
+from quodlibet import const
+
+from quodlibet.formats._audio import AudioFile
+from quodlibet.qltk.remote import FSInterface
+from quodlibet.player.nullbe import NullPlayer
 
 class TFSInterface(TestCase):
-    from formats._audio import AudioFile as AF
     def setUp(self):
-        self.w = SongWatcher()
-        self.fs = FSInterface(self.w)
+        self.p = NullPlayer()
+        self.fs = FSInterface(self.p)
 
     def do(self):
         while gtk.events_pending(): gtk.main_iteration()
@@ -18,19 +22,19 @@ class TFSInterface(TestCase):
         self.failIf(os.path.exists(const.CURRENT))
 
     def test_start(self):
-        self.w.song_started(self.AF({"woo": "bar", "~#length": 10}))
+        self.p.emit('song_started', AudioFile({"woo": "bar", "~#length": 10}))
         self.do()
         self.failUnless("woo=bar\n" in file(const.CURRENT).read())
 
     def test_song_ended(self):
-        self.w.song_started(self.AF({"woo": "bar", "~#length": 10}))
+        self.p.emit('song-started', AudioFile({"woo": "bar", "~#length": 10}))
         self.do()
-        self.w.song_ended({}, False)
+        self.p.emit('song-ended', {}, False)
         self.do()
         self.failIf(os.path.exists(const.CURRENT))
 
     def tearDown(self):
-        self.w.destroy()
+        self.p.destroy()
         try: os.unlink(const.CURRENT)
         except EnvironmentError: pass
 add(TFSInterface)
