@@ -1,6 +1,7 @@
-import gtk
 from tests import TestCase, add
-from qltk.wlw import WaitLoadWindow
+
+import gtk
+from quodlibet.qltk.wlw import WaitLoadWindow
 
 class TWaitLoadWindow(TestCase):
     class DummyConnector(gtk.Window):
