@@ -194,6 +194,11 @@ class Thuman_sort(TestCase):
 
         self.failUnlessEqual(self.smaller(u"bbb", u"zzz3"), True)
 
+    def test_false(self):
+        # album browser needs that to sort albums without artist/title
+        # to the bottom
+        self.failIf(util.human_sort_key(""))
+
     def test_white(self):
         self.failUnlessEqual(
             util.human_sort_key(u"  3foo    bar6 42.8"),
@@ -246,20 +251,20 @@ class Tformat_size(TestCase):
         map(self.failUnlessEqual, map(util.format_size, d.keys()), d.values())
 
     def test_bytes(self):
-        self.t_dict({ 0: "0B", 1: "1B", 1023: "1023B" })
+        self.t_dict({ 0: "0 B", 1: "1 B", 1023: "1023 B" })
 
     def test_kbytes(self):
-        self.t_dict({ 1024: "1.00KB", 1536: "1.50KB",
-                      10240: "10KB", 15360: "15KB" })
+        self.t_dict({ 1024: "1.00 KB", 1536: "1.50 KB",
+                      10240: "10 KB", 15360: "15 KB" })
 
     def test_mbytes(self):
-        self.t_dict({ 1024*1024: "1.00MB", 1024*1536: "1.50MB",
-                      1024*10240: "10.0MB", 1024*15360: "15.0MB",
-                      123456*1024: "121MB", 765432*1024: "747MB"})
+        self.t_dict({ 1024*1024: "1.00 MB", 1024*1536: "1.50 MB",
+                      1024*10240: "10.0 MB", 1024*15360: "15.0 MB",
+                      123456*1024: "121 MB", 765432*1024: "747 MB"})
 
     def test_gbytes(self):
-        self.t_dict({ 1024*1024*1024: "1.0GB", 1024*1024*1536: "1.5GB",
-                      1024*1024*10240: "10.0GB", 1024*1024*15360: "15.0GB"})
+        self.t_dict({ 1024*1024*1024: "1.0 GB", 1024*1024*1536: "1.5 GB",
+                      1024*1024*10240: "10.0 GB", 1024*1024*15360: "15.0 GB"})
 add(Tformat_size)
 
 class Tsplit_title(TestCase):
@@ -312,13 +317,19 @@ class Tsplit_people(TestCase):
         self.failUnlessEqual(util.split_people("foo (bar)"), ("foo", ["bar"]))
     def test_with_person(self):
         self.failUnlessEqual(
-            util.split_people("foo (with bar)"), ("foo", ["bar"]))
+            util.split_people("foo (With bar)"), ("foo", ["bar"]))
     def test_with_with_person(self):
         self.failUnlessEqual(
             util.split_people("foo (with with bar)"), ("foo", ["with bar"]))
     def test_featuring_two_people(self):
         self.failUnlessEqual(
             util.split_people("foo featuring bar, qx"), ("foo", ["bar", "qx"]))
+    def test_featuring_person_bracketed(self):
+        self.failUnlessEqual(
+            util.split_people("foo (Ft. bar)"), ("foo", ["bar"]))
+        self.failUnlessEqual(
+            util.split_people("foo(feat barman)"), ("foo", ["barman"]))
+
 add(Tsplit_people)
 
 class Ttag(TestCase):
@@ -390,6 +401,8 @@ class Tpattern(TestCase):
                             "<i>&amp;</i>")
     def test_invalid(self):
         self.failUnlessEqual(util.pattern("<date"), "")
+        util.pattern("<d\\")
+
 add(Tpattern)
 
 class Tformat_time_long(TestCase):
